#Pythonでファイルを自動バックアップしてPC内を大掃除しよう
# https://news.mynavi.jp/techplus/article/zeropython-96/

import os, shutil
from pathlib import Path
from datetime import datetime

# バックアップ先のフォルダを指定(書き換えが必要) --- (*1)
backup_root_dir = Path(r'D:\\') # WindowsでDドライブにバックアップ
# backup_root_dir = Path(r'/Volumes/Extreme SSD/') # macOSで外付けSSDに

# 年月日のフォルダを作る --- (*2)
ymd = datetime.now().strftime('%Y-%m-%d')
backup_dir = backup_root_dir.joinpath(ymd)
backup_dir.mkdir(parents=True, exist_ok=True)
print('[Backup]', backup_dir)

# バックアップを実行する関数 --- (*3)
def my_backup(name, src, des):
    src = Path(src).absolute()
    des = Path(des).absolute()
    print(f'[{name}] {src}')
    copy_ex(src, des, 0)

# 再帰的にファイルをコピー --- (*4)
def copy_ex(src, des, level):
    try:
        if src.is_file():
            shutil.copy2(src, des)
            # print('-', src)
            return    
        des.mkdir(exist_ok=True)
        for f in src.iterdir():
            copy_ex(f, des.joinpath(f.name), level+1)
    except:
        print('[ERROR]', src)

# デスクトップのコピー --- (*5)
desktop_dir = os.path.expanduser('~/Desktop')
my_backup('Desktop', desktop_dir, backup_dir.joinpath('Desktop'))

# ドキュメントをコピー --- (*6)
doc_dir = os.path.expanduser('~/Documents')
my_backup('Documents', doc_dir, backup_dir.joinpath('Documents'))
print('完了')
