#!/usr/bin/perl

use lib 'd:/Programs/Perl/lib';
use lib 'c:/Programs/Perl/lib';
#use lib 'd:/MyWebs/cgi-bin/lib';

use lib '/home/tkamiya/bin/lib';
use lib '/home/tkamiya/bin';

use Utils;
use ProgVars;
use MyApplication;
use Crystal::CIF;
use Crystal::Crystal;
use Crystal::AtomType;

use strict;
#use warnings;


my $ProgramDir      = ProgVars::ProgramDir(); #"d:\\Programs";
my $BinDir          = ProgVars::BinDir();
my $DBDir           = ProgVars::PeriodicTableDir();

my $RietanFPDir         = ProgVars::RietanFPDir();
my $RietanFPProgramPath = Deps::MakePath($RietanFPDir, "RIETAN_VENUS", 0);

my $SpaceGroupDBPath     = Deps::MakePath($RietanFPDir, ["RIETAN_VENUS", "SPGRA"], 0);
my $SPGRDBPath           = Deps::MakePath($RietanFPDir, ["RIETAN_VENUS", "spgr.dat"], 0);
my $ASFDCPath            = Deps::MakePath($RietanFPDir, ["RIETAN_VENUS", "asfdc.dat"], 0);

my $NonrelInputPath           = Deps::MakePath($DBDir, "nonrel",                      0);
my $IonRadius1DBPath          = Deps::MakePath($DBDir, "IonRadius1.html",             0);
my $IonRadius2DBPath          = Deps::MakePath($DBDir, "IonRadius2.html",             0);
my $PeriodicTable1DBPath      = Deps::MakePath($DBDir, "PeriodicTable1.html",         0);
my $PeriodicTable2DBDir       = Deps::MakePath($DBDir, "PeriodicTable2",              0);
my $PeriodicTable3DB          = Deps::MakePath($DBDir, "PERIODIC.CSV",                0);
my $ElectronicConfigurationDB = Deps::MakePath($DBDir, "ElectronicConfiguration.csv", 0);

my $MXDDir     = ProgVars::MXDDir();
my $MXDExecDir = ProgVars::MXDExecDir();
my $SX1DBPath  = ProgVars::SX1DBPath();

my $LF = "\n";

my $OutputDBDir = Deps::MakePath($DBDir, 'atoms', 0);


my $in = JFile->new($NonrelInputPath, 'r') or die "$!";
for(my $i = 1 ; ; $i++) {
# H  HFS ATOM CALC.
#    1  300    1
#   1.00000   0.97804  20.00000  32.00000   0.00000
#  1.0  0.0  0.0   -0.30       1.0
	my $line1 = $in->ReadLine();  #    1  300    1
	last if(!$line1);

	my ($atomname) = ($line1 =~ /^\s*(.*?)\s/);
	$atomname = ucfirst lc $atomname;
	last if($atomname eq '');

	my $DBPath = Deps::MakePath($OutputDBDir, sprintf("%03d-%s.ini", $i, $atomname), 0);
	print("i=$i: $atomname  saved to [$DBPath]\n");

	my ($patomdb, $pmaterialdb, $pbusingdb) = &GetAtomDBHash($atomname, 0);
	my $out = JFile->new($DBPath, 'w') or die $!;
	$out->print("[Atom]\n");
	foreach my $key (sort keys %$patomdb) {
		print("  $key: $patomdb->{$key}\n");
		if($key eq 'pIonRadius') {
			my $p = $patomdb->{$key};
			my $val = join(',', @$p);
#			$out->print("$key=$val\n");
			}
		else {
			$out->print("$key=$patomdb->{$key}\n");
		}
	}
	$out->print("\n");
	$out->print("[Busing]\n");
	foreach my $key (sort keys %$pbusingdb) {
		print("  $key: $pbusingdb->{$key}\n");
		$out->print("$key=$pbusingdb->{$key}\n");
	}
	$out->print("\n");
	$out->print("[Material]\n");
	foreach my $key (sort keys %$pmaterialdb) {
		print("  $key: $pmaterialdb->{$key}\n");
		$out->print("$key=$pmaterialdb->{$key}\n");
	}

	while(1) {
		my $line2 = $in->ReadLine();     #   1.00000   0.97804  20.00000
		last if(!$line2 or Utils::DelSpace($line2) eq '');
	}

# ASFDC
	my ($a1, $a2, $a3) = ReadASFParameters($atomname, $ASFDCPath, 0);
	$out->print("\n");
	$out->print("[ASF]\n");
	for(my $i = 0; $i < @$a1; $i++) {
		print("  a1[$i]: $a1->[$i]\n");
		$out->print("a1[$i]=$a1->[$i]\n");
	}
	for(my $i = 0; $i < @$a2; $i++) {
		print("  a2[$i]: $a2->[$i]\n");
		$out->print("a2[$i]=$a2->[$i]\n");
	}
	for(my $i = 0; $i < @$a1; $i++) {
		print("  a3[$i]: $a3->[$i]\n");
		$out->print("a3[$i]=$a3->[$i]\n");
	}

	$out->Close();
}
$in->Close();

exit;

sub GetAtomicNumber 
{
	my ($name, $Debug) = (@_);

	$name =~ s/\{.*?\}//g;
	$name =~ s/\d//g;
	$name = uc $name;
	$name =~ s/^(\D+?)\d.*$/$1/;
	my $an = -1;
	unless(open(IN, "<$NonrelInputPath")) {
		print "Error: $!: $NonrelInputPath${LF}";# if($Debug);
		return -1;
	}

	while(<IN>) {
# H  HFS ATOM CALC.
#    1  300    1
#   1.00000   0.97804  20.00000  32.00000   0.00000
#  1.0  0.0  0.0   -0.30       1.0
  		if($_ =~ /^\s*$name\s/) { # H  HFS ATOM CALC.
			my $line = <IN>;  #    1  300    1
			$line = <IN>;     #   1.00000   0.97804  20.00000
			($an) = ($line =~ /^\s*(.*?)\s/);
			last;
		}
	}
	close(IN);
	return $an;
}


sub GetAtomDBHash {
	my ($element, $charge, $nCoordination) = @_;
	$charge = 0 if(!defined $charge);

	if($element =~ /^\d+$/) {
		$element = AtomType::GetAtomName($element);
	}

	my ($mass, $charge0, $ai, $bi, $ci, $rad) = AtomType::GetBusingParameter($element);
	my ($AtmNum, $JName, $EName, $NameOrigin) = AtomType::GetIonInfFromPeriodicTable1($element);
	my ($AtomicNumber, $AtomName, $Charge)    = AtomType::GetAtomInformation($element);
	my $CationRadius = AtomType::GetCationRadius($element, $charge, $nCoordination);
	my @IonRadius    = AtomType::GetIonRadius2($element, $charge, $nCoordination);
	my ($filepath, $AtomJName, $AtomEName,
		$AtomicMass, $FoundYear, $AtomicRadius,
		$MeltingPoint, $BoilingPoint, $Density,
		$HeatCapacity, $IonizationPotential, $ElectronAffinity) = AtomType::GetIonInfFromPeriodicTable2($AtmNum);
	my $NameOrigin2;
	($AtmNum, $JName, $EName, $NameOrigin2) = AtomType::GetIonInfFromPeriodicTable1($element);
	my ($pOrbitalName1, $pnElectron1)       = AtomType::GetElectronicConfiguraton($AtmNum);
	my ($pOrbitalName2, $pnElectron2)       = AtomType::SpeculateElectronicConfiguration($AtmNum);
	my $ElectronConfiguration1 = '';
	my $ElectronConfiguration2 = '';
	if($pOrbitalName1) {
		for(my $i = 0 ; $i < @$pOrbitalName1 ; $i++) {
			next if($pnElectron1->[$i] == 0);
			$ElectronConfiguration1 .= "$pOrbitalName1->[$i]($pnElectron1->[$i])";
		}
		for(my $i = 0 ; $i < @$pOrbitalName2 ; $i++) {
			next if($pnElectron2->[$i] == 0);
			$ElectronConfiguration2 .= "$pOrbitalName2->[$i]($pnElectron2->[$i])";
		}
	}
#print "  $ElectronConfiguration1\n";
#print "  $ElectronConfiguration2\n";

	$AtomicMass = $mass        if(!defined $AtomicMass);
	$JName      = $AtomJName   if(!defined $JName);
	$EName      = $AtomEName   if(!defined $EName);
	$NameOrigin = $NameOrigin2 if(!defined $NameOrigin);

	my %atomdb = (
		name                   => $element,
		Z                      => $AtmNum,
		mass                   => $AtomicMass,
		charge                 => $charge,
#		nCoordination          => $nCoordination,
		AtomicRadius           => $AtomicRadius,
#		CationRadius           => $CationRadius,
		pIonRadius             => \@IonRadius,
		ElectronConfiguration1 => $ElectronConfiguration1,
		ElectronConfiguration2 => $ElectronConfiguration2,
		JName                  => $JName,
		EName                  => $EName,
		NameOrigin             => $NameOrigin,
		NameOrigin2 => $NameOrigin2,
		FoundYear              => $FoundYear,
		);

	my %materialdb = (
		IonizationPotential    => $IonizationPotential,
		ElectronAffinity       => $ElectronAffinity,
		MeltingPoint           => $MeltingPoint,
		BoilingPoint           => $BoilingPoint,
		Density                => $Density,
		HeatCapacity           => $HeatCapacity,
		);

	my %busingdb = (
		ai                     => $ai,
		bi                     => $bi,
		ci                     => $ci,
		rad => $rad,
		charge => $charge0,
		);

	return \%atomdb, \%materialdb, \%busingdb;
}

sub ReadASFParameters
{
	my ($AtomName, $asfdcfile, $StopByError) = @_;
	$asfdcfile = $ASFDCPath if(!defined $asfdcfile or $asfdcfile eq '');
	$StopByError = 1 if(!defined $StopByError);
#print "AtomName=[$AtomName]  Path=[$ASFDCPath]\n";

	my $in = new JFile($asfdcfile, "r");
	if(!$in) {
		print "Can not read ASF for [$AtomName] from [$asfdcfile].\n";
exit if($StopByError);
		return undef;
	}

	my $ReadAtomName;
	while(!$in->eof()) {
		my $line = $in->ReadLine();
		last if($in->eof());

		my ($n) = Utils::Split("\\s+", $line);
#		print("n=", $n, "  AtomName=", $AtomName, "\n");
		if(uc $n eq uc $AtomName) { #$line =~ /^\s*$AtomName\s/i) {
			$ReadAtomName = $line;
			last;
		}
	}

#print "Atom: [$AtomName] $ReadAtomName";
	my $line1 = $in->ReadLine();
	my $line2 = $in->ReadLine();
	my @a1 = Utils::Split("\\s+", $line1);
	my @a2 = Utils::Split("\\s+", $line2);
	my @a3 = ();
	if($line2 !~ /0\//) {
		@a3 = Utils::Split("[,\\s]+", $line2);
	}

	$in->Close();

if(!defined $line2 or @a1 < 11) {
	print "Error in Rietan::ReadASFParameters: Can not find ASF for [$AtomName].\n";
	exit if($StopByError);
}

	return \@a1, \@a2, \@a3;
}
