import sys
import numpy
from matplotlib import pyplot as plt
import matplotlib.widgets as wg


from tklib.tkobject import tkObject


class tkPlotEvent(tkObject):
    def __init__(self, plt, **args):
        self.plt = plt
# マウスクリック時に最近接データの情報をコンソールに表示
        self.hit_itarget = None
        self.hit_axisinf = None
        self.stop_flag   = False

        super(tkObject, self).__init__(**args)
#        super(tkParams, self).__init__(parameter_file, app, args)
#        self.update(**args)

        self.axis_inf = []

    def button_click(self, e):
        self.stop_flag = True

    def finalize_button(self, text = 'finished'):
        self.button.label.set_text(text)

    def add_button(self, button_region = [0.15, 0.95, 0.10, 0.03], plot_region = [0.92, 0.15],
                        text = 'stop', color = '#f8e58c', hovercolor = '#38b48b', callback = None):
        if callback is None:
            callback = self.button_click

        self.button_region = button_region
        self.plot_region   = plot_region
        self.color = color
        self.hovercolor = hovercolor
        self.callback = callback
        self.ax_button = plt.axes(button_region)

        self.button = wg.Button(self.ax_button, text, color = color, hovercolor = hovercolor)
        self.button.on_clicked(callback)

        return self.button
    
    def layout(self, show = False):
        self.plt.tight_layout()
        self.plt.subplots_adjust(top = self.plot_region[0], bottom = self.plot_region[1])
        
        if show:
            if use_pause is not None:
                self.plt.pause(use_pause)
            else:
                self.plt.show()

    def remove(self, index):
        if type(index) is int:
            self.axis_inf.pop(index)
            return

        for i in range(len(self.axis_inf)):
            if self.axis_inf[i]['label'] == index:
                self.axis_inf.pop(i)
                return

    def add_data(self, axis_inf = {}):
#{"label": "test predict",     "plot_type": "2D", "axis": axis, "data": test_predict_data,  
# "xlist": [index_test_list, *x_test_list], "xlabels": ["index", *x_labels]})

        plot_type = axis_inf['plot_type']

# プロットの仕方によってデータ構造が違うので、ここで統一しておく
        axis = axis_inf['axis']
        data = axis_inf['data']
        if axis_inf.get('xlabel', None):
            axis.xlabel = axis_inf['xlabel']
        else:
            axis.xlabel = axis.get_xlabel()
        if axis_inf.get('ylabel', None):
            axis.ylabel = axis_inf['ylabel']
        else:
            axis.ylabel = axis.get_ylabel()

        if plot_type == '2D' or plot_type == 'plot':
            for i in range(len(data)):
                data[i].xdata = data[i].get_xdata()
                data[i].ydata = data[i].get_ydata()
        elif plot_type == 'scatter':
            data = [data]
            axis_inf['data'] = data
            data[0].xdata = axis_inf['x']
            data[0].ydata = axis_inf['y']
        else:
            print("")
            print(f"Error in tkplotevent.add_data(): Invalid plot_type [{plot_type}]")
            input(">>")
            exit()

        self.axis_inf.append(axis_inf)
        return axis_inf

    def register_event(self, fig, event = "button_press_event", callback = None):
        if callback is None:
            callback = lambda event: self.onclick(event)

        fig.canvas.mpl_connect(event, callback)
#            fig_contour.canvas.mpl_connect("button_press_event", onclick)

# マウスクリック時にクリックされたグラフ軸を特定
    def find_target_axes(self, event):
        naxis = len(self.axis_inf)
        itargets     = []
        hit_axisinfs = []
        for i in range(naxis):
#            print("a=", event.inaxes, self.axis_inf[i]["axis"])
            if event.inaxes == self.axis_inf[i]["axis"]:
               itargets.append(i)
               hit_axisinfs.append(self.axis_inf[i])

        if len(itargets) == 0:
            return None, None

        return itargets, hit_axisinfs

# マウスクリック時に最近接データを検索
    def find_nearest_data(self, x, y, itarget_list, hit_axisinf_list):
        minr2     = 1.0e300
        iinf_hit  = None
        idata_hit = None
        ihit      = None
        for iaxis in range(len(hit_axisinf_list)):
            hit_axisinf = hit_axisinf_list[iaxis]
            axis        = hit_axisinf['axis']
            data        = hit_axisinf['data']
            plot_type   = hit_axisinf['plot_type']
            xlim = axis.get_xlim()
            ylim = axis.get_ylim()
            xrange = xlim[1] - xlim[0]
            yrange = ylim[1] - ylim[0]
            if type(data) is not list:
                data = [data]

            for idata in range(len(data)):
                xdata = data[idata].xdata
                ydata = data[idata].ydata
#                xdata = data[idata].get_xdata()
#                ydata = data[idata].get_ydata()
                for i in range(len(xdata)):
                    xi = xdata[i]
                    yi = ydata[i]
#                    print("i=", i, xi, yi, x, y)
                    if xi is None or yi is None:
                        continue

                    dx = (x - xi) / xrange
                    dy = (y - yi) / yrange
                    r2 = dx * dx + dy * dy

                    if minr2 > r2:
                        minr2 = r2
#                        print("\n**iaxis is updated:", iaxis, itarget_list[iaxis])
                        iinf_hit  = itarget_list[iaxis]
                        idata_hit = idata
                        ihit  = i
#                    print("  => ", minr2, ihit)

        if iinf_hit is None:
            print("")
            print(f"Error: Invalid type [{plot_type}] in find_nearest_data()")
            return None, None, None, None

        return iinf_hit, idata_hit, ihit, minr2

    def onclick(self, event):
#        print("click", event.inaxes)
        itargets, hit_axisinfs = self.find_target_axes(event)
        if itargets is None:
            print("Warning in tkplotevent.onclick(): Can not find axes")
            return

        iinf, idata, idx, r2 = self.find_nearest_data(event.xdata, event.ydata, itargets, hit_axisinfs)

        self.display_data(iinf, idata, idx, self.axis_inf[iinf])

# 最近接データの情報をコンソールに表示
    def display_data(self, iinf, idata, idx, hit_axisinf = None):
        if hit_axisinf is None:
            hit_axisinf = self.axisinf[iinf]

        plot_type = hit_axisinf['plot_type']
        label  = hit_axisinf['label']
        axis   = hit_axisinf['axis']
        xlabel = axis.xlabel
        ylabel = axis.ylabel
        data = hit_axisinf['data']
        if type(data) is list:
            data = data[0]

        print("")
        if plot_type == '2D' or plot_type == 'plot' or plot_type == 'scatter':
            xdata   = data.xdata
            ydata   = data.ydata
            xlist   = hit_axisinf.get('xlist', None)
            xlabels = hit_axisinf.get('xlabels', None)
            if xlist is None or xlabels is None:
                print(f"clicked for the data [{label}] in the {xlabel} - {ylabel} plot (inf #{iinf}  idata #{idata}  data index #{idx})")
                print(f"  #{idx:4}: {xlabel:>12}: {xdata[idx]:<12.4g}  {ylabel:>12}: {ydata[idx]:<12.4g}")
            else:
                print(f"clicked for the data [{label}] in the {xlabel} - {ylabel} plot (inf #{iinf}  data index #{idx})")
                print(f"  #{idx:4}: {xlabel:>5}: {xdata[idx]:5}  {ylabel:>12}: {ydata[idx]:<12.4g}")
                nx = len(xlabels)
                for i in range(nx):
                    print(f"    {i}: ", end = '')
                    if type(xlist[i][idx]) is int:
                        print(f"{xlabels[i]:>10}: {xlist[i][idx]:<10d}  ")
                    elif type(xlist[i][idx]) is float or isinstance(xlist[i][idx], numpy.float64):
                        print(f"{xlabels[i]:>10}: {xlist[i][idx]:<10.4g}  ")
                    else:
                        print(f"{xlabels[i]:>10}: {xlist[i][idx]:<10}  ")

