from math import exp, sqrt, log
import numpy as np
from pprint import pprint
import openpyxl

from tklib.tkobject import tkObject
from tklib.tkfile import tkFile
from tklib.tkinifile import tkIniFile
from tklib.tkvariousdata import tkVariousData
from tklib.tkutils import terminate, pint, pfloat, getarg, getintarg, getfloatarg, analyze_varstr
from tklib.tksci.tksci import pi, h, hbar,c, e, kB, me
#from tklib.tksci.tkoptimize import mlsq_general


ekB = e / kB


def split_optstr(str):
    try:
        val, id = str.split(':')
        return pfloat(val), pint(id)
    except:
        return str, None

def strlist(vars, format):
    s = ""
    for i in range(len(vars)):
       s += format.format(vars[i]) + ', '
    return s


class tkMobility(tkObject):
    def __init__(self):
        super(tkObject, self).__init__()

        self.path   = None
        self.header = None
        self.xT     = None
        self.yN     = None
        self.ymu    = None
        self.ys     = None
        self.yS     = None

# 移動度の温度依存性計算の際の規格化温度
        self.Tnorm = 300.0
# Tnormにおける電子移動度
        self.mu0    = None
        self.Eb     = None
        self.Eop    = None
        self.Aop    = None
        self.ppolyi = []
        self.apolyi = []

        self.varkeys = None
        self.varname = None
        self.varunit = None
        self.ai0     = None         # Default values
        self.ai      = None
        self.optid   = None
        self.id_llsq = None

        self.debug   = 0
        self.use_simple = 0
        
    def __del__(self):
#        super(tkObject, self).__del__()
        pass

    def __str__(self):
        return self.ClassPath()


    def set(self, **args):
        self.update(args)

    def set_misc_parameters(self, params):
        pass
#        self.Einteg0             = params.get("Einteg0",             self.Einteg0)

    def recover_parameters(self, ais, optid = None, aidef = None):
        if optid is None:
            optid = self.optid
        if aidef is None:
            aidef = self.ai0

        ai = []
        c = 0
        for i in range(len(optid)):
            if optid[i] == 1:
                ai.append(ais[c])
                c += 1
            else:
                ai.append(aidef[i])

        return ai

    def choose_parameters(self, ais0, optid = None):
        if optid is None:
            optid = self.optid

        ais = ais0.copy()
#ai0     = [mu0, Eb, Eop, a1, a2]
    
        ai = []
        c = 0
        for i in range(len(optid)):
            if optid[i] == 1:
                ai.append(ais[i])

        return ai

    def parameter_list(self):
        list = []
        nvars = len(self.varkeys)
        for i in range(nvars):
            key = self.varkeys[i]
            list.append(self.get2(key))
        return list
    
    def parameter_tuple(self):
        return tuple(self.parameter_list())

    def set_parameters(self, ai = None):
        if ai is None:
            ai = self.ai

        for i in range(len(self.varkeys)):
            key = self.varkeys[i]
#            print("set: key=", key, ai[i])
            self.set_attribute2(key, ai[i])

    def read_parameters(self, path, optid = None):
        if optid is None:
            optid = self.optid

        ini = tkIniFile()
        inf = ini.ReadAll(path, AddSection = 0)
        if inf is None:
            return

#        print("optid=", optid)
#        print("self.varkeys=", self.varkeys)
        for i in range(len(self.varkeys)):
            key = self.varkeys[i]
            str = inf.get(key, None)
            val, id = split_optstr(str)
#            print("read: ", key, val, id)
            if val is not None:
                self.set_attribute2(key, pfloat(val))
            if id is not None:
                optid[i] = id
        
    def save_parameters(self, path, optid = None,
                    otherparams = None, parameters_section = 'Parameters', other_section = 'OtherParameters'):
        if optid is None:
            optid = self.optid

        ini = tkIniFile(path)
        for i in range(len(self.varkeys)):
            key = self.varkeys[i]
            val = self.get2(key, None)
            if val is not None:
#                print("save key=", key, val, optid[i])
                ini.WriteString(parameters_section, key, "{}:{}".format(val, optid[i]))

        if otherparams is not None:
            for key in otherparams.keys():
                val = otherparams[key]
                if val is not None:
                    ini.WriteString(other_section, key, val)

    def print_parameters(self, varname = None, varunit = None, ai = None, optid = None):
        if varname is None:
            varname = self.varname
        if varunit is None:
            varunit = self.varunit
        if ai is None:
            ai = self.parameter_list()
        if optid is None:
            optid = self.optid

#        print("self.varkeys=", self.varkeys)
        for i in range(len(self.varkeys)):
            key = self.varkeys[i]
            val = self.get2(key, None)
            if val is not None:
                print("  {:10s}: {:14.8g}  optid={}".format(key, pfloat(val), optid[i]))
            else:
                print("  {:10s}: None  optid={}".format(key, optid[i]))

    def mlsq_general3(self, xT, ymuinv, lsqfunc, iPrint = 1):
        print("tkMobility.mlsq_general3:")

        ndata = len(xT)
        m = 3
        print("  nData =", ndata)
        print("      xT=", strlist(xT,  "{:10.4g}"))
        print("  ymuinv=", strlist(ymuinv, "{:10.4g}"))

        Si  = np.zeros([m, 1])
#        for ci in range(m):
#            Si[ci, 0] = sum([ymuinv[idata] * lsqfunc(ci, xT[idata]) for idata in range(ndata)])
        for idata in range(ndata):
            for ci in range(m):
                Si[ci, 0] += ymuinv[idata] * lsqfunc(ci, xT[idata])
    
        Sij = np.zeros([m, m])
#        for ci in range(m):
#             for cj in range(ci, m):
#                 v = sum([lsqfunc(ci, xT[idata]) * lsqfunc(cj, xT[idata]) for idata in range(ndata)])
#                 Sij[ci, cj] = Sij[cj, ci] = v
        for ci in range(m):
             for cj in range(m):
                 for idata in range(ndata):
                     T = xT[idata]
                     Sij[ci, cj] += lsqfunc(ci, T) * lsqfunc(cj, T)

        ai = np.linalg.inv(Sij) @ Si
        ai = ai.transpose().tolist()
        ai = ai[0]
        

        """
        print("")
        print("1/mu vs lsq: lsq_general3")
        print("{:8}\t{:10}\t{:10}".format("T", "1/mu", "cal"))
        ycal = []
        for idata in range(ndata):
            T = xT[idata]
            v = 0.0
            for ci in range(m):
                v += ai[ci] * lsqfunc(ci, T)
            ycal.append(v)
            print("{:8.4g}\t{:10.4g}\t{:10.4g}".format(T, ymuinv[idata], ycal[idata]))
#        exit()
        """
        
        if iPrint == 1:
            print("tkoptimize.mslq_general3:: Vector and Matrix:") 
            print("Si=")
            pprint(Si)
            print("Sij=")
            pprint(Sij)
            print("ai=")
            pprint(ai)
            print("")

        apolyi = self.apolyi.copy()
        for i in range(3):
            apolyi[i] = ai[i]

        aalli  = [self.ai0[0]]
        ntotalvars = len(self.ai0)
        ci = 0
        for ivar in range(1, ntotalvars):
            if self.optid[ivar] == 1:
                aalli.append(ai[ci])
                ci += 1
            else:
                aalli.append(self.ai0[ivar])

        return ai, apolyi, aalli, Si, Sij

    def linearfit3(self, Tlist, mulist):
        muinvlist = []
        for i in range(len(mulist)):
#            muinvlist.append(1.0 / mulist[i])
            muinvlist.append(pow(Tlist[i], -0.5) * exp(-self.Eb * ekB / Tlist[i]) / mulist[i])

        ai, aipoly, aiall, Si, Sij = self.mlsq_general3(Tlist, muinvlist, self.lsqfunc, iPrint = 1)
        self.apolyi = ai

#        """
        print("")
        print("1/mu vs lsq: linearfit3")
        print("{:8}\t{:10}\t{:10}\t{:10}\t{:10}".format("T", "1/mu", "cal", "mu", "cal"))
        ndata = len(Tlist)
        m = 3
        ycal = []
        for idata in range(ndata):
            T = Tlist[idata]
            muinv = 0.0
            for ci in range(m):
                muinv += ai[ci] * self.lsqfunc(ci, T)
            ycal.append(muinv)
#            mu = pow(T, -0.5) * exp(-self.Eb * ekB / T) / muinv
            mu = self.cal_mu(T)
            print("{:8.4g}\t{:10.4g}\t{:10.4g}\t{:10.4g}\t{:10.4g}".format(T, muinvlist[idata], ycal[idata], mulist[idata], mu))
#        exit()
#        """
        
        return ai, aipoly, aiall, Si, Sij

# Calculate inverse of scattering term multiplied by T^0.5
    def lsqfunc(self, idx, T):
#        t = T / self.Tnorm
        t = T

        if idx == 0:
            if T == 0.0:
                return 0.0
            else:
                k = self.Eop * e / kB / T
                if k > 70.0:
                    return 0.0
                else:
                    return 1.0 / (exp(k) - 1.0)
        else:
            ip = idx - 1
            if ip >= len(self.ppolyi):
                 nfunc = int((len(self.varkeys) - 1) / 2)
                 terminate("Too many functions required (idx = {}, nfunc={})".format(idx, nfunc))

            y = pow(t, -self.ppolyi[ip])
#            print("ip=", ip, self.ppolyi[ip], self.apolyi[ip], y)

            return y

    def cal_mu_components(self, T):
        if self.use_simple == 1:
            npoly = 4
        else:
            npoly = int(len(self.varkeys) / 2 + 0.00001)

        Kgb  = exp(-self.Eb * ekB / T)

        muinv_tot = 0.0
        mus  = []
        for i in range(npoly):
            if i == 0:
                muinv = self.Aop * self.lsqfunc(0, T)
                if muinv < 1.0e-10:
                    muop = 1.0e10
                else:
                    muop = 1.0 / muinv
            else:
                ip = i - 1
                muinv = self.apolyi[ip] * self.lsqfunc(i, T)
                if abs(muinv) < 1.0e-10:
                    mus.append(1.0e10)
                else:
                    mus.append(1.0 / muinv)
#                print("   mu=", 1.0 / muinv, "  muinv=", muinv, "  ip=", ip, self.apolyi[ip])

            muinv_tot += muinv
#            print("**muinv_tot=", i, muinv_tot, muinv)

        mu_ingrain = 1.0 / muinv_tot
        mu_tot = Kgb * mu_ingrain
#        print("mu_tot(0)=", mu_ingrain, muop, *mus)
        
        return mu_tot, Kgb, mu_ingrain,muop, mus

    def cal_mu(self, T): #, ai = None):
#        if ai is None:
#            ai = self.ai

        mu_tot, Kgb, mu_ingrain, muop, mus = self.cal_mu_components(T)
        return mu_tot

    def mlsq_general(self, xT, ymuinv, lsqfunc, iPrint = 1):
#        self.debug = 1

        print("")
        print("tkMobility.mlsq_general:")
        ndata      = len(xT)
        ntotalvars = len(self.ai0)

# 最適化するパラメータを抽出
        m = 0
        vars = []
        for i in range(1, ntotalvars):
            if self.id_llsq[i] == 1 and self.optid[i] == 1:
                m += 1
                vars.append(self.varkeys[i])

        print("  nTotalVars:", ntotalvars)
        print("        Vars:", strlist(self.varkeys, "{:>10}"))
        print("      Values:", strlist(self.ai0,     "{:10.4g}"))
        print("  id        :", strlist(self.optid,   "{:10d}"))
        print("  id_llsq   :", strlist(self.id_llsq, "{:10d}"))
        print("    nOptVars:", m)
        print("     OptVars:", vars)
        print("  nData     :", ndata)

# 最適化する1/mu - 1/mu,fixを計算。mu,fixは、線形最小自乗に入れない他の項の和
        yconv = []
        for idata in range(ndata):
            T = xT[idata]
            ymuinvfix = 0.0
            for ivar in range(1, ntotalvars, 2):
                if self.id_llsq[ivar] == 0 or self.optid[ivar] == 0:
                    ifunc = int((ivar - 1) / 2 + 0.00001)
#                    print("ivar=", ivar, ifunc, self.ai0[ivar])
                    ymuinvfix += self.ai0[ivar] * lsqfunc(ifunc, T)

            yconv.append(ymuinv[idata] - ymuinvfix)

#        print("            xT=", strlist(xT,  "{:10.4g}"))
#        print("        ymuinv=", strlist(ymuinv, "{:10.4g}"))
#        print("  ymuinv(conv)=", strlist(yconv, "{:10.4g}"))
        print("")
        print("{:8}\t{:12}\t{:12}\t{:12}".format("T(K)", "mu(cm2/Vs)", "1/mu(Vs/cm2)", "1/mu - 1/mu,fix"))
        for iT in range(ndata):
            print("{:8.3f}\t{:12.4g}\t{:12.4g}\t{:12.4g}".format(xT[iT], 1.0 / ymuinv[iT], ymuinv[iT], yconv[iT]))

        Si  = np.zeros([m, 1])
        for idata in range(ndata):
            T = xT[idata]
            ci = 0
            for ivar in range(1, ntotalvars):
                if self.optid[ivar] == 1 and self.id_llsq[ivar] == 1:
                    ifunc = int((ivar - 1) / 2)
                    Si[ci, 0] += yconv[idata] * lsqfunc(ifunc, T)
                    ci += 1
#        print("Si=", Si)
    
        Sij = np.zeros([m, m])
        ci = 0
        for ivar in range(1, ntotalvars, 2):
            if self.optid[ivar] == 1 and self.id_llsq[ivar] == 1:
                ifunc = int((ivar - 1) / 2)
                cj = 0
                for jvar in range(1, ntotalvars, 2):
                    if self.optid[jvar] == 1 and self.id_llsq[ivar] == 1:
                        jfunc = int((jvar - 1) / 2)

                        for idata in range(ndata):
                            T = xT[idata]
                            Sij[ci, cj] += lsqfunc(ifunc, T) * lsqfunc(jfunc, T)

                        cj += 1
                ci += 1

        ai = np.linalg.inv(Sij) @ Si
        ai = ai.transpose().tolist()
        ai = ai[0]

        if iPrint == 1:
            print("")
            print("tkoptimize.mslq_general:: Vector and Matrix:") 
            print("Si=")
            pprint(Si)
            print("Sij=")
            pprint(Sij)
            print("ai=")
            pprint(ai)
            print("")

        aalli  = [self.ai0[0]]
        ci = 0
        for ivar in range(1, ntotalvars):
            if self.optid[ivar] == 1 and self.id_llsq[ivar] == 1:
                aalli.append(ai[ci])
                ci += 1
            else:
                aalli.append(self.ai0[ivar])

        apolyi = []
        ci = 0
        for ivar in range(1, ntotalvars, 2):
            if self.optid[ivar] == 1 and self.id_llsq[ivar] == 1:
                apolyi.append(ai[ci])
                ci += 1
            else:
                apolyi.append(self.ai0[ivar])

        if self.debug:
            print("")
            print("1/mu vs lsq: lsq_general")
            print("{:8}\t{:10}\t{:10}".format("T", "1/mu", "cal"))
            ycal = []
            for idata in range(ndata):
                T = xT[idata]
                v = 0.0
                for ci in range(m):
                    v += apolyi[ci] * lsqfunc(ci, T)
                ycal.append(v)
                print("{:8.4g}\t{:10.4g}\t{:10.4g}".format(T, ymuinv[idata], ycal[idata]))

        return ai, apolyi, aalli, Si, Sij

    def linearfit(self, Tlist, mulist):
        muinvlist = []
        for i in range(len(mulist)):
#            muinvlist.append(1.0 / mulist[i])
#            muinvlist.append(pow(Tlist[i], -0.5) * exp(-self.Eb * ekB / Tlist[i]) / mulist[i])
            muinvlist.append(exp(-self.Eb * ekB / Tlist[i]) / mulist[i])


        ai, apolyi, aalli, Si, Sij = self.mlsq_general(Tlist, muinvlist, self.lsqfunc, iPrint = 1)
        self.apolyi = apolyi

        if self.debug:
            print("")
            print("apolyi=", apolyi)
            print("aalli=", aalli)
            print("1/mu vs lsq: linearfit")
            print("{:8}\t{:10}\t{:10}\t{:10}\t{:10}".format("T", "1/mu", "cal", "mu", "cal"))
            ndata = len(Tlist)
            m = len(apolyi)
            ycal = []
            for idata in range(ndata):
                T = Tlist[idata]
                muinv = 0.0
                for ci in range(m):
                    muinv += apolyi[ci] * self.lsqfunc(ci, T)
                ycal.append(muinv)
#                muobs = pow(T, -0.5) * exp(-self.Eb * ekB / T) / muinvlist[idata]
                muobs = mulist[idata]
#                muv = pow(T, -0.5) * exp(-self.Eb * ekB / T) / muinv
                muv = self.cal_mu(T)
                print("{:8.4g}\t{:10.4g}\t{:10.4g}\t{:10.4g}\t{:10.4g}".format(T, muinvlist[idata], ycal[idata], muobs, muv))
#            exit()

        return ai, apolyi, aalli, Si, Sij

    def read_excel_general(self, infile, 
                 T_label = r'^T[$|\s|\(|\[]', n_label = r'^N[$|\S]?.*$', 
                 mu_label = r'^mu[$|\S]?.*$', sigma_label = r'^sigma[$|\S]?.*$',
                 S_label = r'^S[$|\S]?.*$', usage = None):
        datafile = tkVariousData(infile)
        labels, datalist = datafile.Read_minimum_matrix(close_fp = True, usage = usage)
        label_sample, xsample = datafile.FindDataArray(r'^[^(S|sigma|N|mu)][$|\S]?.*$')

        label_T, xT           = datafile.FindDataArray(T_label)
        label_sigma, ysigma   = datafile.FindDataArray(sigma_label, flag = 'i')
        label_N, yN           = datafile.FindDataArray(n_label, flag = 'i')
        label_mu, ymu         = datafile.FindDataArray(mu_label, flag = 'i')
        label_S, yS           = datafile.FindDataArray(S_label)

        n = len(xT)
        if ysigma is None:
            ysigma = [1.0e-10] * n
        if yN is None:
            yN = [1.0e15] * n
        if ymu is None:
            ymu = [1.0e-3] * n
        if yS is None:
            yS = [1.0e-6] * n

        inf = { "label_T":     label_T,     "T":     xT,
                "label_sigma": label_sigma, "sigma": ysigma,
                "label_N":     label_N,     "N":     yN,
                "label_mu":    label_mu,    "mu":    ymu,
                "label_S":     label_S,     "S":    yS,
              }
        self.header = [label_T, label_sigma, label_N, label_mu, label_S]
        self.xT  = xT
        self.yN  = yN
        self.ymu = ymu
        self.ys  = ysigma
        self.yS  = yS

        return inf
        
    def read_excel(self, infile):
        wb = openpyxl.load_workbook(infile, data_only = True)
        if not wb:
            print("")
            print("Error to read [{}]".format(infile))
            print("")
            terminate()

        sheetnames = wb.sheetnames
        print("sheet names:", sheetnames)
        print("  read [{}]".format(sheetnames[0]))

#       ws = wb.active
        ws = wb[sheetnames[0]]
        Tlabel  = ws[1][0].value
        Nlabel  = ws[1][1].value
        mulabel = ws[1][2].value
        slabel  = ws[1][3].value
        print("Labels:", Tlabel, Nlabel, slabel, mulabel)
        xT  = []
        yN  = []
        ymu = []
        ys  = []
        i = 2
        while 1:
            Tcell = ws.cell(row = i, column = 1).value
            if Tcell is None or Tcell == '':
                break

            Ncell  = ws.cell(row = i, column = 2).value
            mucell = ws.cell(row = i, column = 3).value
            scell  = ws.cell(row = i, column = 4).value
#            print("cells=", Tcell, Ncell, scell, mucell)

            Tcell  = float(Tcell)
            Ncell  = float(Ncell)
            scell  = float(scell)
            mucell = float(mucell)

            xT.append(Tcell)
            yN.append(Ncell)
            ys.append(scell)
            ymu.append(mucell)
            i += 1

        self.header = [Tlabel, Nlabel, mulabel, slabel]
        self.xT  = xT
        self.yN  = yN
        self.ymu = ymu
        self.ys  = ys

        return self.header, xT, yN, ymu, ys


def main():
    print("")
    print("This is a library, not runnable")
    print("")


if __name__ == "__main__":
    main()
