#===============================================
# PVAnalyze
#===============================================
package PVAnalyze;
use TkFittingCommon;
@ISA = qw(TkFittingCommon);

use strict;

use Utils;
use IniFile;
use CSV;

use MyTk::GraphFrameArray;
use GraphData;

use Sci qw($pi $kB $c $e $e0 $me $mn $h $hbar $torad $todeg);
use Sci::Material;
use Sci::SolarCell;
use Sci::DiodeRsParallelRsRshJpv;
use Sci::Optimize;

#============================================================
#　Global変数
#============================================================
#my $material = new Material;
my $EntryWidth    = 12;
my $TextBoxWidth  = 30;
my $TextBoxHeight = 20;

#============================================================
#　変数等取得関数
#============================================================

#============================================================
#　コンストラクタ、デストラクタ
#============================================================
sub new
{
	my ($class, $app, @a) = @_;

	my $self = TkFittingCommon->new($app, @a);
	my $this = bless $self, $class;
#	$this->SetApplication($app) if($app);

	return $this;
}

sub DESTROY
{
	my $this = shift;
	$this->SUPER::DESTROY(@_);
}


#============================================================
#　継承クラスで定義しなおす関数
#============================================================
sub CreateWidgets
{
	my ($this) = @_;
	my $App    = $this->App();
	my $args   = $App->Args();

	$this->{ini}  = new IniFile;
	$this->{dini} = new IniFile;
	$this->InitializeParameters($this->{ini}, $this->{dini});

	$this->SUPER::CreateWidgets(0, 1);

#	$this->SetTitle("$Program / TkPlot");
#	$this->App()->SetProgram("$Program / TkPlot");

#===================================================
# ペインを作製
#===================================================
	my $Frame = $this->MyFrame()->pack(-anchor => 'nw', -fill => 'x');
	$this->MakeChooseFileEntry($Frame, 'Param', 'Param:', $this->{ini}->pVariable("ParameterFile", ""), 
				'&Choose', sub { $this->ChooseFile(@_); }, "ChooseParameterFile");

	$this->{SaveParamButton} = $Frame->MyButton(
			-text => '&Save',
			-takefocus => 1,
			-command => [\&ButtonPressed, $this, 'RButtonDown', 'SaveParam'],
		)->pack(-side => 'left');
	$this->{EditParamButton} = $Frame->MyButton(
			-text => 'ed',
			-takefocus => 1,
			-command => [\&ButtonPressed, $this, 'RButtonDown', 'EditParam'],
		)->pack(-side => 'left');

	$Frame = $this->MyFrame()->pack(-anchor => 'nw', -fill => 'x');
	$this->MakeChooseFileEntry($Frame, 'CSV',    'CSV:', $this->{ini}->pVariable("DataFile", ""),
				'&Choose', sub { $this->ChooseFile(@_); }, "ChooseCSVFile");
	$this->{EditDataButton} = $Frame->MyButton(
			-text => 'ed',
			-takefocus => 1,
			-command => [\&ButtonPressed, $this, 'RButtonDown', 'EditData'],
		)->pack(-side => 'left');

	$Frame = $this->MyFrame()->pack(-anchor => 'nw', -fill => 'x');
	my $lsb1 = $this->{ChooseXListBox} = $Frame->MyBrowseEntry(
			-label      => "X:",
			-state      => "readonly",
			-takefocus  => 1,
			-width      => 5,
			-Selections => [],
			-SelIndex   => 0,
		)->pack(-side => 'left', -expand => 'yes', -fill => 'x');
	$lsb1->configure(-browsecmd  => [\&SelChangeListBox, $this, "X", $this->{ChooseXListBox}]);
	my $lsb2 = $this->{ChooseY1ListBox} = $Frame->MyBrowseEntry(
			-label      => "Y1:",
			-state      => "readonly",
			-takefocus  => 1,
			-width      => 5,
			-Selections => [],
			-SelIndex   => 1,
		)->pack(-side => 'left', -expand => 'yes', -fill => 'x');
	$lsb2->configure(-browsecmd  => [\&SelChangeListBox, $this, "Y1", $this->{ChooseY1ListBox}]);

	$Frame = $this->MyFrame()->pack(-anchor => 'nw', -fill => 'x');
	$this->MakeLabelEntry($Frame, "FitXMin", "XMin:", 
			$this->{ini}->pVariable('FitXMin', ''), "%s", $EntryWidth, "", 0, 0);
	$this->MakeLabelEntry($Frame, "FitXMin", "XMax:", 
			$this->{ini}->pVariable('FitXMax', ''), "%s", $EntryWidth, "", 0, 1);
	$this->MakeLabelEntry($Frame, "nSkipData", "nSkip:", 
			$this->{ini}->pVariable('nSkipData', 0), "%d", $EntryWidth, "", 0, 2);
	$this->MakeLabelEntry($Frame, "EPS", "EPS:", 
			$this->{ini}->pVariable('EPS', 1.0e-5), "%12.4g", $EntryWidth, "", 1, 0);
	$this->MakeLabelEntry($Frame, "nMaxIter", "MaxIter:", 
			$this->{ini}->pVariable('nMaxIter', 1000), "%d", $EntryWidth, "", 1, 1);
	$this->MakeLabelEntry($Frame, "nSaveSpectraInterval", "SaveIntvl:", 
			$this->{ini}->pVariable('nSaveSpectraInterval', 10), "%d", $EntryWidth, "", 1, 2);

	$Frame = $this->MyFrame()->pack(-anchor => 'nw', -fill => 'x');
	$this->{ChooseLSQMethodListBox} = $Frame->MyBrowseEntry(
			-label      => "LSQ Method:",
			-state      => "readonly",
			-takefocus  => 1,
			-width      => 5,
			-Selections => ['Amoeba::Simplex', 'PDL::Simplex', 'ModifiedNewton'],
			-SelIndex   => 0,
		)->pack(-side => 'left', -expand => 'yes', -fill => 'x');

	my $lsb3 = $this->{ChooseModelListBox} = $Frame->MyBrowseEntry(
			-label      => "Model:",
			-state      => "readonly",
			-takefocus  => 1,
			-width      => 5,
			-Selections => ["Polynomial", "Ohmic/SCLC", "SymForwardSchottky", "SymReverseSchottky", "SymReverseNDSchottky"],
			-SelIndex   => 0,
		)->pack(-side => 'left', -expand => 'yes', -fill => 'x');
	$lsb2->configure(-browsecmd  => [\&SelChangeListBox, $this, "Model", $this->{ChooseModelListBox}]);

	$Frame = $this->MyFrame()->pack(-anchor => 'nw');
	$Frame->MyCheckbutton(
		-text     => 'Use Dark',
		-takefocus => 1,
		-variable => $this->{ini}->pVariable('UseDark', 1),
		-onvalue  => 1,
		-offvalue => 0,
		-indicatoron => 1,
		)->pack(-side => 'left', -fill => 'x');

	$Frame = $this->MyFrame()->pack(-anchor => 'nw');
	$this->MakeLabelEntry($Frame, "T",     "T:", $this->{ini}->pVariable('T', 300.0), "%g",     $EntryWidth, "K",      0, 0);
	$this->MakeLabelEntry($Frame, "F",     "F:", $this->{ini}->pVariable('F', 100.0), "%g",     $EntryWidth, "cm-2",   0, 1);

	$Frame = $this->MyFrame()->pack(-anchor => 'nw');
	$this->MakeLabelEntry($Frame, "Voc",  "Voc:", $this->{ini}->pVariable('Voc', 0.0), "%6.4g", $EntryWidth, "V",     0, 0);
	$this->MakeLabelEntry($Frame, "Jsc",  "Jsc:", $this->{ini}->pVariable('Jsc', 0.0), "%6.4g", $EntryWidth, "Acm-2", 0, 1);
	$this->MakeLabelEntry($Frame, "FF",   "FF:",  $this->{ini}->pVariable('FF',  0.0), "%6.4g", $EntryWidth, "",      0, 2);

	$Frame = $this->MyFrame()->pack(-anchor => 'nw');
	$this->MakeLabelEntry($Frame, "Eff",  "Eff:", $this->{ini}->pVariable('Eff', 0.0), "%6.4g", $EntryWidth, "%",     0, 0);

	$Frame = $this->MyFrame()->pack(-anchor => 'nw');
	$this->MakeLabelEntry($Frame, "Vop",  "Vop:",  $this->{ini}->pVariable('Vop', 0.0),  "%6.4g", $EntryWidth, "V",     0, 0);
	$this->MakeLabelEntry($Frame, "Jop",  "Jop:",  $this->{ini}->pVariable('Jop', 0.0),  "%6.4g", $EntryWidth, "Acm-2", 0, 1);
	$this->MakeLabelEntry($Frame, "Wmax", "Wmax:", $this->{ini}->pVariable('Wmax', 0.0), "%6.4g", $EntryWidth, "Wcm-2", 0, 2);

	$Frame = $this->MyFrame()->pack(-anchor => 'nw');
	$this->MakeLabelEntry($Frame, "Jsat",    "Jsat:",    $this->{ini}->pVariable('Jsat', 0.0),  "%6.4g", $EntryWidth, "",        0, 0);
	$this->MakeLabelEntry($Frame, "Rs(on)",  "Rs(on):",  $this->{ini}->pVariable('RsOn', 0.0),  "%6.4g", $EntryWidth, "",        0, 1);
	$this->MakeLabelEntry($Frame, "Rs(Voc)", "Rs(Voc):", $this->{ini}->pVariable('RsVoc', 0.0), "%6.4g", $EntryWidth, "ohm cm2", 0, 2);

	$Frame = $this->MyFrame()->pack(-anchor => 'nw');
	$this->MakeLabelEntry($Frame, "Rsh(0)",   "Rsh(0):",   $this->{ini}->pVariable('Rsh0', 0.0),  "%6.4g", $EntryWidth, "", 0, 0);
	$this->MakeLabelEntry($Frame, "Rsh(rev)", "Rsh(rev):", $this->{ini}->pVariable('RsRev', 0.0), "%6.4g", $EntryWidth, "ohm cm2", 0, 1);

	$Frame = $this->MyFrame()->pack(-anchor => 'nw', -fill => 'x');
	$this->{AnalyzePVButton} = $Frame->MyButton(
			-text => '&Analyze PV',
			-takefocus => 1,
			-command => [\&ButtonPressed, $this, 'RButtonDown', 'AnalyzePV'],
		)->pack(-side => 'left');

	$Frame = $this->MyFrame()->pack(-anchor => 'nw', -fill => 'x');
	$this->{TextBox} = $Frame->Scrolled(
		'MyText',
#		-readonly   => 1,
		-width      => $TextBoxWidth,
		-height     => $TextBoxHeight,
		-scrollbars => 'e',
		-takefocus  => 1,
	)->pack(-side => 'top', -expand => 'yes', -fill => 'both');
#	if($App->{FileContentFontName}) {
#		$this->{FileContentText}->configure(
#			-font => [ $App->{FileContentFontName},
#				   $App->{FileContentFontSize},
#				   $App->{FileContentFontStyle} ],
#		);
#	}

	$this->UpdateParameters();

#ツールバーのOpenボタンをファイル読み込みにバインドする
	$this->{OpenButton}->configure(
		-command  => [ \&ChooseFile, $this, $this->{SPEFileFileEntry} ],
		) if($this->{OpenButton});

	$this->InitWindowPosition($App->{geometory});
}

sub InsertToolBar
{
	my ($this, $frame) = @_;
#ツールバーに追加
	my $spb1 = $this->{ShowRecalcDataButton} = $frame->MyCheckbutton(
			-text => 'Show Recalc data',
			-takefocus => 1,
			-variable => $this->{ini}->pVariable('ShowRecalcData', 1),
			-indicatoron => 0,
		)->pack(-side => 'left', -fill => 'x');
	my $spb2 = $this->mw()->{ShowCalcPlotButton} = $frame->MyCheckbutton(
			-text => 'Show CalcPlot',
			-takefocus => 1,
			-variable => $this->{ini}->pVariable('ShowCalcPlot', 1),
			-indicatoron => 0,
		)->pack(-side => 'left', -fill => 'x');
	my $spb3 = $this->mw()->{AutoUpdateButton} = $frame->MyCheckbutton(
			-text => 'Auto Update',
			-takefocus => 1,
			-variable => $this->{ini}->pVariable('AutoUpdate', 0),
			-indicatoron => 0,
		)->pack(-side => 'left', -fill => 'x');
}

sub ButtonPressed
{
	my ($this, $event, $type) = @_;
	my $App = $this->App();
	if($type eq 'Recalc') {
		&Recalc($this);
#		$this->CreateGraphFrame();
		$this->AssignGraphData(0);
		$this->Draw();
		return;
	}
	elsif($type eq 'AnalyzePV') {
		&AnalyzePV($this);
#		$this->CreateGraphFrame();
		$this->AssignGraphData(0);
		$this->Draw();
		return;
	}
	elsif($type eq 'Fit') {
		&Fit($this);
#		$this->CreateGraphFrame();
		$this->AssignGraphData(0);
		$this->Draw();
		return;
	}
	elsif($type eq 'SaveParam') {
		$this->{ini}->{Model} = $this->{ChooseModelListBox}->GetText();
		$this->{ini}->{X}     = $this->{ChooseXListBox}->GetText();
		$this->{ini}->{Y1}    = $this->{ChooseY1ListBox}->GetText();
		$this->{ini}->SetIniFile($this->{ini}->{ParameterFile}, undef, 1);
		if(!defined $this->{ini}->IniFile() or $this->{ini}->IniFile() eq '') {
			my $path = $this->ChooseSaveFile('save.prm', '*.prm');
			if($path) {
				$this->{ini}->SetIniFile($path, undef, 1);
				$this->{ini}->{ParameterFile} = $path;
			}
			else {
				return;
			}
		}
		$this->{ini}->WriteAll();
		return;
	}
	elsif($type eq 'EditParam') {
		my $Editor  = $this->App()->{EditorPath};
		my $File = $this->{ini}->{ParameterFile};
		my $command = "$Editor $File";
		system($command);
		return;
	}
	elsif($type eq 'EditData') {
		my $Editor  = $this->App()->{EditorPath};
		my $File = $this->{ini}->{DataFile};
		my $command = "$Editor $File";
		system($command);
		return;
	}
}

sub UpdateParameters
{
	my ($this) = @_;

if(0) {
	my $mater = new Material;
	my $Dc0 = $mater->Dc0($this->{ini}->{mes}*$me, 1.0);
	$this->{ini}->{NCs} = $mater->Nc($this->{ini}->{mes}*$me, 1.0, $this->{ini}->{T}) * 1.0e-6;
	$this->{ini}->{EFs} = $this->{ini}->{ECs} + $mater->CalEFFromNe($this->{ini}->{Nes}*1.0e6, $this->{ini}->{T});
#	$this->{ini}->{EFs} = $mater->BMShift($this->{ini}->{Nes}*1.0e6, $Dc0);

	my ($R, $Reff) = $mater->RichardsonDushman($this->{ini}->{mem}*$me);
	$this->{ini}->{Am} = $Reff;
#	$this->{ini}->{Vd} = -($this->{ini}->{EFs} - $this->{ini}->{EFm});
#	$this->{ini}->{phiSBH} = -($this->{ini}->{ECs} - $this->{ini}->{EFm});

	$this->{ini}->{bs} = sqrt($e / 4.0 / $pi / $e0 / $this->{ini}->{Epss});

	my $T = $this->{ini}->{T};
	my $ekT = $e / $kB / $T;
	$this->{ini}->{Is0} = $this->{ini}->{Am} * $T * $T * exp(-$this->{ini}->{phiSBH} * $ekT) * $this->{ini}->{S} * 1.0e-4;
}

	my $pEA = $this->{pEntryArray};
	for(my $i = 0 ; $i < @$pEA ; $i++) {
		$pEA->[$i]->Update();
	}
}

sub EntryFocusedOut {
	my ($this, $obj, $event, $type) = @_;
print "EntryFocusedOut: type=$type\n";

if(0) {
	my $mater = new Material;
	if($event eq 'FocusOut') {
		if($type eq 'Am') {
			$this->{ini}->{mem} = $mater->CalEffectiveMassFromRichardsonDushman($this->{ini}->{Am}) / $me;
		}
		elsif($type eq 'Vd') {
			$this->{ini}->{EFm} = $this->{ini}->{EFs} + $this->{ini}->{Vd};
		}
		elsif($type eq 'phiSBH') {
			$this->{ini}->{ECs} = $this->{ini}->{EFm} - $this->{ini}->{phiSBH};
		}
	}
	if($this->{ini}->{AutoUpdate} and defined $this->{ini}->{"${type}check"}) {
		&Recalc($this);
		$this->CreateGraphFrame() if(!defined $this->{GraphFrameArray});
		$this->AssignGraphData(0);
		$this->Draw();
	}
	else {
		$this->UpdateParameters();
	}
}
}

#===================================================
# データ処理
#===================================================
sub AnalyzePV
{
	my ($this) = @_;
	my $ini = $this->{ini};

	$this->UpdateParameters();

	my $iXLabel  = $this->{"ChooseXListBox"}->GetCurSel();
	my $iY1Label = $this->{"ChooseY1ListBox"}->GetCurSel();
	my $ppDataArray = $this->{ppDataArray};
	my $pV  = $ppDataArray->[$iXLabel];
	my $pY1 = $ppDataArray->[$iY1Label];
	my $nData = @$pV;
	my $MaxI = 0.0;
	for(my $i = 0 ; $i < $nData ; $i++) {
		my $absI = abs($pY1->[$i]);
		$MaxI = $absI if($MaxI < $absI);
	}

	my $FitXMin     = $ini->{FitXMin};
	my $FitXMax     = $ini->{FitXMax};
	my $nSkipData   = $ini->{nSkipData};
	my $Method      = $this->{ChooseLSQMethodListBox}->GetText();
	my $EPS         = $ini->{EPS};
	my $nMaxIter    = $ini->{nMaxIter};
	my $iPrintLevel = $ini->{iPrintLevel};





my ($nSimplexIter, $RsId, $Method, $EPS, $nMaxIter, $iPrintLevel);

	$this->{TextBox}->SetText('');

	my $UseDark = $ini->{UseDark};
	my $DoLSQ   = $ini->{DoLSQ};
	my $F       = $ini->{F};
	my $T       = $ini->{T};
	my $nSkip   = $ini->{nSkip};
	$nSkip = 1 if($nSkip <= 0);

my ($ILabel, $JLabel);
if($UseDark) {
	$ILabel  = "I.*";
#	$ILabel  = "Id.*";
	$JLabel  = "J.*";
#	$FitVMin = -0.5;
#	$FitVMax =  3.0;
}
else {
	$ILabel  = "I.*";
#	$ILabel  = "Ip.*";
	$JLabel  = "J.*";
#	$FitVMin = -3.0;
#	$FitVMax =  5.0;
#	$ILabel  = ($UseDark)? "Id.*" : "Ip.*";
}

my $SourceCSV = $this->{Path};
my ($drive, $directory, $filename, $ext1, $lastdir, $filebody) = Deps::SplitFilePath($SourceCSV);
my $OutTXT    = Deps::MakePath("$drive$directory", "${filebody}-Out.txt", 0);
my $OutCSV    = Deps::MakePath("$drive$directory", "${filebody}-Out.csv", 0);
my $FitCSV    = Deps::MakePath("$drive$directory", "${filebody}-Fit.csv", 0);

my $txt = new JFile;
if(!$txt->Open($OutTXT, "w")) {
	print "$!: Can not write to [$OutTXT].\n";
	return 0;
}
$txt->mprint("Source   : $SourceCSV\n");
$txt->mprint("OutCSV   : $OutCSV\n");
$txt->mprint("FitCSV   : $FitCSV\n");
$txt->mprint("For dark?: $UseDark\n");
$txt->mprint("Current data taken from: $ILabel\n");
$txt->mprint("Light intensity: $F mW/cm2\n");
$txt->mprint("Temperature    : $T K\n");

my $SC = new SolarCell;
$SC->SetLightIntensity($F);
$SC->SetTemplature($T);
my $iData = 0;
my ($prevV, $VIndex);
my $VLabel = "V.*";
my ($pV, $pI, $pJ, $nData) = $SC->ReadIVData($SourceCSV, $VLabel, $ILabel, $JLabel, 3, 
	sub {
		my ($csv, $pLabelList, $pDataArray, $pCur) = @_;
		if(!defined $VIndex) {
			$VIndex = $csv->FindLabelIndex([$VLabel], $pLabelList);
		}
		if(!defined $VIndex) {
			print "Error: Can not find V column data.\n";
			exit;
		}
		my $V = $pCur->[$VIndex];
#print "$iData for Vidx=$VIndex: $V\n";
		$prevV = $V - 1.0e-5 if(!defined $prevV);
		if($V <= $prevV) {
			return -1;
		}
		$prevV = $V;
		$iData++;
		return 1;
	}
	);
print("nData=$nData\n");
if(!$pJ) {
	print "Error: Can not find the data column I\n";
	exit;
}
if($nData == 0) {
	print "Error: nData = 0\n";
	exit;
}
my $prevV = $pV->[0] - 1.0e-5;
for(my $i = 0 ; $i < $nData ; $i++) {
	if($pV->[$i] <= $prevV) {
		$SC->{nData} = $i;
		last;
	}
	print "$i\t$pV->[$i]\t$pJ->[$i]\n";
	$prevV = $pV->[$i];
}

$SC->Analyze(1);
if(!$SC->WriteAnalyzeCSV($OutCSV)) {
	print "$!: Can not write to [$OutCSV]\n";
	return 0;
}

$txt->mprint("\n");
$txt->mprint("Source data\n");
$txt->mprint("  nData=$SC->{nData}\n");
$txt->mprint("  Electrode area: $SC->{Area} cm2\n");

foreach my $key qw(Jsc Voc Jsat Vop Jop Wmax FF Eff RsOn RsVoc Rsh0 RsRev VforR VrevR) {
	$ini->{$key}= $SC->{$key};
}
$this->UpdateParameters();

$txt->mprint("\n");
$txt->mprint("Electric parameters\n");
$txt->mprint("Jsc      = $SC->{Jsc} A/cm2\n");
$txt->mprint("Voc      = $SC->{Voc} mA/cm2\n");
$txt->mprint("Jsat     = $SC->{Jsat} mA/cm2 (at V=$SC->{Vsat} V)\n");
$txt->mprint("Vop      = $SC->{Vop} V\n");
$txt->mprint("Jop      = $SC->{Jop} mA/cm2\n");
$txt->mprint("Wmax     = $SC->{Wmax} mW/cm2\n");
$txt->mprint("FF       = $SC->{FF}\n");
$txt->mprint("Eff      = $SC->{Eff} %\n");
$txt->mprint("Rs(on)   = $SC->{RsOn} ohm cm2 (V=$SC->{VforR} V)\n");
$txt->mprint("Rs(Voc)  = $SC->{RsVoc} ohm cm2\n");
$txt->mprint("Rsh(0)   = $SC->{Rsh0} ohm cm2\n");
$txt->mprint("Rsh(rev) = $SC->{RsRev} ohm cm2 (V=$SC->{VrevR} V)\n");

$txt->Close();

	$this->{TextBox}->SetText(JFile->new()->ReadFile($OutTXT));
	return 1 if(!$DoLSQ);

if(!$txt->Open($OutTXT, "a")) {
	print "$!: Can not write to [$OutTXT].\n";
	return 0;
}

my $Rs  = $SC->{RsOn};
my $Rsh = $SC->{RsRev};
my $Jpv = ($UseDark)? 0 : $SC->{Jpv};

my $J01 = $SC->{J0} / 2.0;
my $n1  = $SC->{n};
my $Rs1 = $Rs * 2.0;
my ($J02, $n2, $Rs2) = ($J01, $n1+1.0, $Rs1);
my $Rs  = 0.0;

$txt->mprint("\n");
$txt->mprint("Intial values:\n");
$txt->mprint("J01 = $J01 A/cm2\n");
$txt->mprint("n1  = $n1\n");
$txt->mprint("Rs1 = $Rs1 ohm cm2\n");
$txt->mprint("J02 = $J02 A/cm2\n");
$txt->mprint("n2  = $n2\n");
$txt->mprint("Rs2 = $Rs2 ohm cm2\n");
$txt->mprint("Rs  = $Rs ohm cm2\n");
$txt->mprint("Rsh = $Rsh ohm cm2\n");
$txt->mprint("Jpv = $Jpv A/cm2\n");

my ($OptVars, $MinVal);
my @VarLabel  = ("J01", "n1", "Rs1", "J02", "n2", "Rs2",  "Rs", "Rsh", "Jpv");
for(my $i = 0 ; $i < $nSimplexIter ; $i++) {
	my @Guess = ( $J01,  $n1,  $Rs1, $J02,   $n2,  $Rs2,   $Rs, $Rsh,  $Jpv);
	my @OptId = (    1,    1,     1,    1,     1,     1, $RsId,    1,     1);
$OptId[@OptId-1] = 0 if($UseDark);
	my $dRs   = ($Rs == 0.0)?    0.1 : $Rs * 0.1;
	my $dJpv  = ($Jpv == 0.0)? 0.001 : $Jpv * 0.1;
	my @Scale = ($J01*0.1, $n1*0.1, $dRs, $J02*0.1, $n2*0.1, $dRs, $dRs, $Rsh*0.1, $dJpv);
	my $Opt = new Optimize;
	($OptVars, $MinVal) = $Opt->Optimize($Method, \@Guess, \@OptId, \@Scale, 
				$EPS, $nMaxIter, $iPrintLevel,
				sub { &MinimizationFunc(@_); },
				sub { },
				sub { Optimize::BuildDifferentialMatrixes(@_); },
				);
	($J01, $n1, $Rs1, $J02, $n2, $Rs2, $Rs, $Rsh, $Jpv) = @$OptVars;
	print "Optimized at (",join(',',@{$OptVars}),")=$MinVal\n";
}

$txt->mprint("\n");
$txt->mprint("Optimized parameters\n");
for(my $i = 0 ; $i < @VarLabel ; $i++) {
	$txt->mprint("$VarLabel[$i]: $OptVars->[$i]\n");
}
$txt->mprint("MinS: $MinVal\n");

my $out = new JFile;
$out->Open($FitCSV, "w") or die "$!: Can not write to [$FitCSV]\n";
$out->print("V(V),log(|Jcal|),log(|Jobs|),|Jcal|(A/cm2),|Jobs|(A/cm2),Jcal(A/cm2),Jobs(A/cm2),Vdiode(V)\n");
for(my $i = 0 ; $i < $nData ; $i++) {
	my $Vtarget = $pV->[$i];
	my ($Vdiode, $Jtotal) = &CalSCFTotalIV($Vtarget, $J01, $n1, $Rs1, $J02, $n2, $Rs2, $Rs, $Rsh, $Jpv);
#print "$Vtarget, $Vdiode, $Jtotal\n";
	my $absJcal = abs($Jtotal);
	my $absJobs = abs($pJ->[$i]);
	my $logJcal = log($absJcal) / log(10.0);
	my $logJobs = log($absJobs) / log(10.0);
	if($i < @VarLabel) {
		$out->print("$Vtarget,$logJcal,$logJobs,$absJcal,$absJobs,$Jtotal,$pJ->[$i],$Vdiode,"
					."$VarLabel[$i]=,$OptVars->[$i]\n");
	}
	elsif($i == @VarLabel) {
		$out->print("$Vtarget,$logJcal,$logJobs,$absJcal,$absJobs,$Jtotal,$pJ->[$i],$Vdiode,"
					."MinS=,$MinVal\n");
	}
	else {
		$out->print("$Vtarget,$logJcal,$logJobs,$absJcal,$absJobs,$Jtotal,$pJ->[$i],$Vdiode\n");
	}
}
$out->Close();
$txt->Close();

	$this->{TextBox}->SetText(JFile->new()->ReadFile($OutTXT));

	return 1;
}

#==============================================
# 最小化関数のサブルーチン
# MinimizationFunc    : 
# PDLDisplaySimplex: PDL::Opt::Simplex用のDisplay関数
#==============================================
sub MinimizationFunc {
	my ($pVars, $iPrintLevel) = @_;
	
	my ($nSkip, $pV, $pJ, $T, $nData, $FitVMin, $FitVMax);
	my ($J01, $n1, $Rs1, $J02, $n2, $Rs2, $Rs, $Rsh, $Jpv) = @$pVars;
	my $S = 0.0;
	my $c = 0;
	for(my $i = 0 ; $i < $nData ; $i++) {
		next if($i % $nSkip != 0);
		my $Vtarget = $pV->[$i];
		next if($Vtarget < $FitVMin or $FitVMax < $Vtarget);

		my ($Vdiode, $Jtotal) = &CalSCFTotalIV($Vtarget, $J01, $n1, $Rs1, $J02, $n2, $Rs2, $Rs, $Rsh, $Jpv);
		if($Jtotal != 0.0 and $pJ->[$i] != 0.0) {
			if($Jtotal * $pJ->[$i] < 0.0) {
				my $absd = abs(($Jtotal-$pJ->[$i]) / $pJ->[$i]);
				$S += (log($absd))**2;
			}
			else {
				my $absd = abs($Jtotal / $pJ->[$i]);
				$S += (log($absd))**2;
			}
		}
#print "$i: $Vtarget, $Jtotal, $pJ->[$i]\n";
		$c++;
	}
	$S += 1.0e5 if($J01 <= 0.0);
	$S += 1.0e5 if($n1  <= 0.0);
	$S += 1.0e5 if($Rs1 < 0.0);
	$S += 1.0e5 if($J02 <= 0.0);
	$S += 1.0e5 if($n2  <= 0.0);
	$S += 1.0e5 if($Rs2 < 0.0);
	$S += 1.0e5 if($Rs  < 0.0);
	$S += 1.0e5 if($Rsh < 0.0);
	$S += 1.0e5 if($Jpv < 0.0);
if($iPrintLevel >= 2) {
	printf("(J0,n,Rs)2,Rs,Rsh,Jpv\n");
	printf("  (%5.3g, %4.3f, %5.3g)\n", $J01, $n1, $Rs1);
	printf("  (%5.3g, %4.3f, %5.3g), %6.3f, %6.3f, %6.3g (%5.3g)\n", $J02, $n2, $Rs2, $Rs, $Rsh, $Jpv, $S);
}

	return $S;
}

sub CalSCFTotalIV
{
	my ($Vtarget, $J01, $n1, $Rs1, $J02, $n2, $Rs2, $Rs, $Rsh, $Jpv) = @_;

	my $T;
	my $dd = new DiodeRsParallelRsRshJpv;
	$dd->SetTemperature($T);
	return $dd->CalSCFTotalIV($Vtarget, $J01, $n1, $Rs1, $J02, $n2, $Rs2, $Rs, $Rsh, $Jpv);
}

sub Fit
{
	my ($this) = @_;
	my $ini = $this->{ini};

	$this->UpdateParameters();

	my $iXLabel  = $this->{"ChooseXListBox"}->GetCurSel();
	my $iY1Label = $this->{"ChooseY1ListBox"}->GetCurSel();
	my $ppDataArray = $this->{ppDataArray};
	my $pV  = $ppDataArray->[$iXLabel];
	my $pY1 = $ppDataArray->[$iY1Label];
	my $nData = @$pV;
	my $MaxI = 0.0;
	for(my $i = 0 ; $i < $nData ; $i++) {
		my $absI = abs($pY1->[$i]);
		$MaxI = $absI if($MaxI < $absI);
	}

#	my $Opt = new Optimize;
	my $optimize = new Optimize;
	$this->{Optimize} = $optimize;

	my $FitXMin    = $ini->{FitXMin};
	my $FitXMax    = $ini->{FitXMax};
	my $nSkipData  = $ini->{nSkipData};
	my $Method     = $this->{ChooseLSQMethodListBox}->GetText();
	my $EPS        = $ini->{EPS};
	my $nMaxIter   = $ini->{nMaxIter};
	my $iPrintLevel = $ini->{iPrintLevel};

	my $Model  = $this->{"ChooseModelListBox"}->GetText();
	if($Model eq 'Polynomial') {
		my ($pName, $ppVariable, $pGuess, $pOptId, $pScale) = $optimize->AddParameters(
			"c0", \$this->{ini}->{c0}, $this->{ini}->{c0check}, 1,
			"C1", \$this->{ini}->{c1}, $this->{ini}->{c1check}, 1,
			"C2", \$this->{ini}->{c2}, $this->{ini}->{c2check}, 1,
			"C3", \$this->{ini}->{c3}, $this->{ini}->{c3check}, 1,
			);
	}
	if($Model eq 'Ohmic/SCLC') {
		my ($pName, $ppVariable, $pGuess, $pOptId, $pScale) = $optimize->AddParameters(
			"sigma", \$this->{ini}->{sigma}, $this->{ini}->{sigmacheck}, $MaxI*10.0,
			"mus",   \$this->{ini}->{mus},   $this->{ini}->{c1check},    1.0,
			);
	}
	elsif($Model eq 'SymForwardSchottky') {
		my ($pName, $ppVariable, $pGuess, $pOptId, $pScale) = $optimize->AddParameters(
			"phiSBH", \$this->{ini}->{phiSBH}, $this->{ini}->{phiSBHcheck}, 1,
			"ndiode", \$this->{ini}->{ndiode}, $this->{ini}->{ndiodecheck}, 1,
			);
	}
	elsif($Model eq 'SymReverseSchottky') {
		my ($pName, $ppVariable, $pGuess, $pOptId, $pScale) = $optimize->AddParameters(
			"phiSBH", \$this->{ini}->{phiSBH}, $this->{ini}->{phiSBHcheck}, 1,
			"ndiode", \$this->{ini}->{ndiode}, $this->{ini}->{ndiodecheck}, 1,
			"kbs",    \$this->{ini}->{kbs},    $this->{ini}->{kbscheck},    1,
			);
	}
	elsif($Model eq 'SymReverseNDSchottky') {
		my $lnNDs = log($this->{ini}->{NDs});
		my ($pName, $ppVariable, $pGuess, $pOptId, $pScale) = $optimize->AddParameters(
			"phiSBH", \$this->{ini}->{phiSBH}, $this->{ini}->{phiSBHcheck}, 1,
			"ndiode", \$this->{ini}->{ndiode}, $this->{ini}->{ndiodecheck}, 1,
			"kbs",    \$this->{ini}->{kbs},    $this->{ini}->{kbscheck},    1,
			"lnNDs",  \$lnNDs,                 $this->{ini}->{NDscheck},   20,
			"Vd",     \$this->{ini}->{Vd},     $this->{ini}->{Vdcheck},     1,
			);
	}

	my ($OptVars, $MinVal) = $optimize->Optimize(
				$Method, undef, undef, undef,
				$EPS, $nMaxIter, $iPrintLevel,
				sub { $this->CalS2($FitXMin, $FitXMax, $nSkipData, @_); },
				undef,
				sub { Optimize::BuildDifferentialMatrixes(@_); },
				);
print "\nOptimized at S=$MinVal:\n";
my $pNames = $optimize->pParameterNameArray();
for(my $i = 0 ; $i < @$OptVars ; $i++) {
	print "$i:[$pNames->[$i]] $OptVars->[$i]\n";
}

	my $IsPrint = 0;
	if($Model eq 'Polynomial') {
		($this->{ini}->{c0}, $this->{ini}->{c1}, $this->{ini}->{c2}, $this->{ini}->{c3}) = @$OptVars;
		($this->{pVcal}, $this->{pIcal}) 
				= $this->CalI($Model, $pV, $this->{ini}->{c0}, $this->{ini}->{c1}, $this->{ini}->{c2}, $this->{ini}->{c3}, $IsPrint);
	}
	elsif($Model eq 'Ohmic/SCLC') {
		($this->{ini}->{sigma}, $this->{ini}->{mus}) = @$OptVars;
		($this->{pVcal}, $this->{pIcal}) 
				= $this->CalI($Model, $pV, $this->{ini}->{sigma}, $this->{ini}->{mus}, $IsPrint);
	}
	elsif($Model eq 'SymForwardSchottky') {
		($this->{ini}->{phiSBH}, $this->{ini}->{ndiode}) = @$OptVars;
		$this->{ini}->{ECs} = $this->{ini}->{EFm} - $this->{ini}->{phiSBH};
		($this->{pVcal}, $this->{pIcal}) 
				= $this->CalI($Model, $pV, $this->{ini}->{T}, $this->{ini}->{phiSBH}, $this->{ini}->{ndiode}, $IsPrint);
	}
	elsif($Model eq 'SymReverseSchottky') {
		($this->{ini}->{phiSBH}, $this->{ini}->{ndiode}, $this->{ini}->{kbs}) = @$OptVars;
		$this->{ini}->{ECs} = $this->{ini}->{EFm} - $this->{ini}->{phiSBH};
		($this->{pVcal}, $this->{pIcal}) 
				= $this->CalI($Model, $pV, $this->{ini}->{T}, $this->{ini}->{phiSBH}, $this->{ini}->{ndiode}, $this->{ini}->{kbs}, $IsPrint);
	}
	elsif($Model eq 'SymReverseNDSchottky') {
		my $lnNDs;
		($this->{ini}->{phiSBH}, $this->{ini}->{ndiode}, $this->{ini}->{kbs}, $lnNDs, $this->{ini}->{Vd}) = @$OptVars;
		$this->{ini}->{NDs} = exp($lnNDs);
		$this->{ini}->{ECs} = $this->{ini}->{EFm} - $this->{ini}->{phiSBH};
		($this->{pVcal}, $this->{pIcal}) 
				= $this->CalI($Model, $pV, $this->{ini}->{T}, 
							$this->{ini}->{phiSBH}, $this->{ini}->{ndiode}, $this->{ini}->{kbs}, $lnNDs, 
							$this->{ini}->{Vd}, $IsPrint);
	}

	print "\nOptimized at:\n";
#	$optimize->PrintParameters(1);

#	$optimize->RecoverParameters($OptVars);

#	$this->SetS2($MinVal);

	$this->UpdateParameters();
}

sub CalS2
{
	my ($this, $FitXMin, $FitXMax, $nSkipData, $pVars, $iPrintLevel) = @_;
	my $optimize = $this->{Optimize};
	my $ini      = $this->{ini};

	my $iXLabel     = $this->{"ChooseXListBox"}->GetCurSel();
	my $iY1Label    = $this->{"ChooseY1ListBox"}->GetCurSel();
	my $ppDataArray = $this->{ppDataArray};
	my $pV  = $ppDataArray->[$iXLabel];
	my $pY1 = $ppDataArray->[$iY1Label];
	my $nData = @$pV;
	return 1.0e10 unless($pV);

#	$optimize->RecoverParameters($pVars);
	$optimize->IncrementnS2();
	my $nS2         = $optimize->nS2Calculation();
	my $nS2Interval = $ini->{nSaveSpectraInterval};

	my $Model  = $this->{"ChooseModelListBox"}->GetText();
	if($Model eq 'Polynomial') {
		my ($c0, $c1, $c2, $c3) = @$pVars;
		($this->{pVcal}, $this->{pIcal}) 
				= $this->CalI($Model, $pV, $c0, $c1, $c2, $c3, 0);
	}
	elsif($Model eq 'Ohmic/SCLC') {
		my ($sigma, $mus) = @$pVars;
		($this->{pVcal}, $this->{pIcal}) 
				= $this->CalI($Model, $pV, $sigma, $mus, 0);
	}
	elsif($Model eq 'SymForwardSchottky') {
		my ($phiSBH, $n) = @$pVars;
		($this->{pVcal}, $this->{pIcal}) 
				= $this->CalI($Model, $pV, $this->{ini}->{T}, $phiSBH, $n, 0);
	}
	elsif($Model eq 'SymReverseSchottky') {
		my ($phiSBH, $n, $kbs) = @$pVars;
		($this->{pVcal}, $this->{pIcal}) 
				= $this->CalI($Model, $pV, $this->{ini}->{T}, $phiSBH, $n, $kbs, 0);
	}
	elsif($Model eq 'SymReverseNDSchottky') {
		my ($phiSBH, $n, $kbs, $lnNDs, $Vd) = @$pVars;
		($this->{pVcal}, $this->{pIcal}) 
				= $this->CalI($Model, $pV, $this->{ini}->{T}, $phiSBH, $n, $kbs, $lnNDs, $Vd, 0);
	}

	my $MaxI = 0.0;
	my $S2 = 0.0;
	my $c = 0;
	$pV = $this->{pVcal};
	$nData = @$pV;
	for(my $i = 0 ; $i < $nData ; $i++) {
#print "$i: $pY1->[$i], $this->{pIcal}->[$i]\n";
		my $V = $pV->[$i];
		my $r = ($pV->[$i] - $FitXMin) / ($FitXMax - $FitXMin);
		next if($r < 0.0 or $r > 1.0);
		next if($nSkipData > 1 and $i % $nSkipData != 0);

		my $dif = $pY1->[$i] - $this->{pIcal}->[$i];
		$S2 += $dif * $dif;
		my $absI = abs($pY1->[$i]);
		$MaxI = $absI if($MaxI < $absI);
		$c++;
	}
	$c = 1 if($c <= 0);
	$S2 = $S2 / $c / $MaxI / $MaxI;

	print "S2=$S2\n";
#	$optimize->PrintParameters(1, $pVars, $S2, 0);

	if($nS2Interval > 0 and $nS2 % $nS2Interval == 0) {
#		$this->SetS2($S2, 0);
		$this->Recalc();
#		$this->CreateGraphFrame();
		$this->AssignGraphData(0);
		$this->Draw();
	}
	$this->UpdateParameters();
	$this->mw()->update();

	return $S2;
}

sub Recalc
{
	my ($this) = @_;
	return if(!$this->{ini}->{ShowCalcPlot});

	$this->UpdateParameters();

	my $iXLabel  = $this->{"ChooseXListBox"}->GetCurSel();
	my $ppDataArray = $this->{ppDataArray};
	my $pV  = $ppDataArray->[$iXLabel];
	return unless($pV);

	my $Model  = $this->{"ChooseModelListBox"}->GetText();
	my $Show = $this->{ini}->{ShowRecalcData};
	if($Model eq 'Polynomial') {
		($this->{pVcal}, $this->{pIcal}) 
				= $this->CalI($Model, $pV, $this->{ini}->{c0}, $this->{ini}->{c1}, $this->{ini}->{c2}, $this->{ini}->{c3}, $Show);
	}
	elsif($Model eq 'Ohmic/SCLC') {
		($this->{pVcal}, $this->{pIcal}) 
				= $this->CalI($Model, $pV, $this->{ini}->{sigma}, $this->{ini}->{mus}, $Show);
	}
	elsif($Model eq 'SymForwardSchottky') {
		($this->{pVcal}, $this->{pIcal}) 
				= $this->CalI($Model, $pV, $this->{ini}->{T}, $this->{ini}->{phiSBH}, $this->{ini}->{ndiode}, $Show);
	}
	elsif($Model eq 'SymReverseSchottky') {
		($this->{pVcal}, $this->{pIcal}) 
				= $this->CalI($Model, $pV, $this->{ini}->{T}, $this->{ini}->{phiSBH}, $this->{ini}->{ndiode}, $this->{ini}->{kbs}, $Show);
	}
	elsif($Model eq 'SymReverseNDSchottky') {
		my $lnNDs = log($this->{ini}->{NDs});
		($this->{pVcal}, $this->{pIcal}) 
				= $this->CalI($Model, $pV, $this->{ini}->{T}, 
					$this->{ini}->{phiSBH}, $this->{ini}->{ndiode}, $this->{ini}->{kbs}, $lnNDs, $this->{ini}->{Vd}, $Show);
	}
}

sub CalI
{
	my ($this, $Model, $IsPrint) = @_;
	my ($pV, $T, $phiB, $n, $kbs, $lnNDs, $Vd);
	my ($c0, $c1, $c2, $c3);
	my ($sigma, $mus);
	if($Model eq 'Polynomial') {
		($this, $Model, $pV, $c0, $c1, $c2, $c3, $IsPrint) = @_;
	}
	elsif($Model eq 'Ohmic/SCLC') {
		($this, $Model, $pV, $sigma, $mus, $IsPrint) = @_;
	}
	elsif($Model eq 'SymForwardSchottky') {
		($this, $Model, $pV, $T, $phiB, $n, $IsPrint) = @_;
	}
	elsif($Model eq 'SymReverseSchottky') {
		($this, $Model, $pV, $T, $phiB, $n, $kbs, $IsPrint) = @_;
	}
	elsif($Model eq 'SymReverseNDSchottky') {
		($this, $Model, $pV, $T, $phiB, $n, $kbs, $lnNDs, $Vd, $IsPrint) = @_;
	}
#print "Model: $Model IsPrint: $IsPrint\n";

	my $ini        = $this->{ini};
	my $FitXMin    = $ini->{FitXMin};
	my $FitXMax    = $ini->{FitXMax};
	my $nSkipData  = $ini->{nSkipData};

	my (@Vcal, @Ical);
	my $nData = @$pV;
#	my $T = $this->{ini}->{T};
	my $S = $this->{ini}->{S} * 1.0e-4; # m2
	my $c = 0;
	if($Model eq 'Polynomial') {
		for(my $i = 0 ; $i < $nData ; $i++) {
			my $V = $pV->[$i];
			next if($nSkipData > 1 and $i % $nSkipData != 0);
			next if($V < $FitXMin or $FitXMax < $V);
			$Vcal[$c] = $V;
			$Ical[$c] = ($c0 + $c1 * $V + $c2 * $V**2 + $c3 * $V**3) * $S;
			print "$V: $Ical[$c]\n" if($IsPrint);
			$c++;
		}
	}
	elsif($Model eq 'Ohmic/SCLC') {
		my $K1 = $sigma * $this->{ini}->{S} / $this->{ini}->{ds};
		my $K2 = 9.0/8.0 * ($mus * 1.0e-4) * ($e0 * $this->{ini}->{Epss}) / ($this->{ini}->{ds} * 1.0e-2)**3 * $this->{ini}->{S};
		for(my $i = 0 ; $i < $nData ; $i++) {
			my $V = $pV->[$i];
			next if($nSkipData > 1 and $i % $nSkipData != 0);
			next if($V < $FitXMin or $FitXMax < $V);

			my $sign = 1.0;
			$sign = -1.0 if($V < 0.0);
			$Vcal[$c] = $V;
			$Ical[$c] = $K1 * $V + $sign * $K2 * $V*$V;
			print "$V: $Ical[$c]\n" if($IsPrint);
			$c++;
		}
	}
	elsif($Model eq 'SymForwardSchottky') {
		my $ekT  = $e / $kB / $T;
		my $K    = $this->{ini}->{Am} * $T * $T * exp(-$phiB * $ekT);
		my $enkT = $e / $kB / $T / $n;
		for(my $i = 0 ; $i < $nData ; $i++) {
			my $V = $pV->[$i];
			next if($nSkipData > 1 and $i % $nSkipData != 0);
			next if($V < $FitXMin or $FitXMax < $V);

			$Vcal[$c] = $V;
			$Ical[$c] = $K * $S * (exp($V * $enkT) - exp(-$V * $enkT)) / 2.0;
			print "$V: $Ical[$c]\n" if($IsPrint);
			$c++;
		}
	}
	elsif($Model eq 'SymReverseSchottky') {
		my $ekT    = $e / $kB / $T;
		my $ekbskT = $this->{ini}->{bs} * $kbs * $ekT;
		my $K      = $this->{ini}->{Am} * $T * $T * exp(-$phiB * $ekT);
		my $enkT   = $e / $kB / $T / $n;
		my $d      = $this->{ini}->{ds} * 1.0e-2; # m
		for(my $i = 0 ; $i < $nData ; $i++) {
			my $V = $pV->[$i];
			next if($nSkipData > 1 and $i % $nSkipData != 0);
			next if($V < $FitXMin or $FitXMax < $V);

			$Vcal[$c] = $V;
			my $E = $V / $d;
			my $I = $S * $K * (exp($ekbskT * sqrt(abs($E))) - 1.0);
			if($V >= 0.0) {
				$Ical[$c] = $I;
			}
			else {
				$Ical[$c] = -$I;
			}
			print "$V: $Ical[$c]\n" if($IsPrint);
			$c++;
		}
	}
	elsif($Model eq 'SymReverseNDSchottky') {
		my $ekT    = $e / $kB / $T;
		my $ekbskT = $this->{ini}->{bs} * $kbs * $ekT;
		my $K      = $this->{ini}->{Am} * $T * $T * exp(-$phiB * $ekT);
		my $enkT   = $e / $kB / $T / $n;
		my $ND     = exp($lnNDs) * 1.0e6; # m-3
		my $er     = $this->{ini}->{Epss};
		my $Ke     = $e / $e0 / $er;
		my $Kc     = sqrt(2.0)/4.0/$pi;
#print "Ke=$Ke, $e, $e0, $er, Kc=$Kc\n";
#print "Ke^1.5=", $Ke**1.5, "\n";
		my $Kexp   = $ekT * sqrt( $Ke**1.5 * $Kc * sqrt($ND));
#print "ekT=$ekT, n=$n, enkT=$enkT\n";
#print "Kexp=$Kexp, ND=$ND m-3\n";
		for(my $i = 0 ; $i < $nData ; $i++) {
			my $V = $pV->[$i];
			next if($nSkipData > 1 and $i % $nSkipData != 0);
			next if($V < $FitXMin or $FitXMax < $V);

			$Vcal[$c] = $V;
			if($V >= 0.0) {
				if($Vd + $V >= 0.0) {
					$Ical[$c] = -$S * $K * exp( $Kexp * ($Vd + $V)**0.25 ) * (exp(-$V * $ekT) - 1.0);
				}
				else {
					$Ical[$c] = -$S * $K * $Kexp * (exp(-$V * $ekT) - 1.0);
				}
			}
			else {
				if($Vd - $V >= 0.0) {
					$Ical[$c] = $S * $K * exp( $Kexp * ($Vd - $V)**0.25 ) * (exp($V * $ekT) - 1.0);
				}
				else {
					$Ical[$c] = $S * $K * $Kexp * (exp($V * $ekT) - 1.0);
				}
			}
			print "$V: $Ical[$c]\n" if($IsPrint);
			$c++;
		}
	}
	return (\@Vcal, \@Ical);
}

sub ReadFile
{
	my ($this, $filepath, $IsPrint, $UpdateRange) = @_;
	$UpdateRange = 1 if(!defined $UpdateRange);
	my $App = $this->App();
#print "type=$type\n";

	$this->{Path} = $filepath;
	$this->{CSV} = new CSV();
	my ($nData, $pLabelArray, @pDataArray) = CSV::GetArraysFromFile($filepath);
#	if(!defined $nData) {
	if(!$this->{CSV}->Read($this->{Path})) {
		$App->print("Error: Can not read [$filepath].\n");
		return $this->{Path} = undef;
	}
#	$this->{pLabelArray} = $pLabelArray;
#	$this->{ppDataArray} = \@pDataArray;
	$this->{pLabelArray} = $this->{CSV}->LabelArray();
	$this->{ppDataArray} = $this->{CSV}->DataArray();

	my $nDataArray = @$pLabelArray;
#print "nDataArray: $nDataArray\n";
	$this->{"ChooseXListBox"}->DeleteAllItem();
	$this->{"ChooseY1ListBox"}->DeleteAllItem();
	for(my $i = 0 ; $i < $nDataArray ; $i++) {
#print "$i: $pLabelArray->[$i]\n";
		$this->{"ChooseXListBox"}->AddItem($pLabelArray->[$i]);
		$this->{"ChooseY1ListBox"}->AddItem($pLabelArray->[$i]);
	}
	$this->{"ChooseXListBox"}->SetCurSel('V', '.*V.*');
	$this->{"ChooseY1ListBox"}->SetCurSel('AJ.*', 'AI.*', 'J.*', 'I.*', '.*J.*', '.*I.*');

	if($UpdateRange or !defined $this->{ini}->{FitXMin}) {
		($this->{ini}->{FitXMin}, $this->{ini}->{FitXMax}) = Utils::CalMinMax($pDataArray[0]);
	}

	$this->CreateGraphFrame();
	$this->AssignGraphData($UpdateRange);
#print "R: $ResetViewRange\n";
	return $filepath;
}

sub CreateGraphFrame
{
	my ($this, $canvas, $TargetData) = @_;
	$canvas = $this->Canvas();
	my $App = $this->App();
	my $font = $App->{'GraphFrameFont'};
	my @font = split(/,/, $font) if($font);
	my $w = $canvas->width();
	my $h = $canvas->height();

	my $GraphFrameArray = $this->{'GraphFrameArray'} = new GraphFrameArray($this->mw());

	$GraphFrameArray->SetCanvasSize($w, $h);

	$GraphFrameArray->AddGraphFrame();
	my $GraphFrame0  = $GraphFrameArray->GetGraphFrame(0);
	my $FramePosStr0 = $App->{"GraphFrame0Position"};
	my $XScale0      = $GraphFrame0->GetXScale(0);
	my $YScale0      = $GraphFrame0->GetYScale(0);
	$GraphFrame0->SetPositionByStr($FramePosStr0);

	$GraphFrameArray->AddGraphFrame();
	my $GraphFrame1  = $GraphFrameArray->GetGraphFrame(1);
	my $FramePosStr1 = $App->{"GraphFrame1Position"};
	my $XScale1      = $GraphFrame1->GetXScale(0);
	my $YScale1      = $GraphFrame1->GetYScale(0);
	$GraphFrame1->SetPositionByStr($FramePosStr1);

	$XScale0->SetScaleStringVisible(1);
	$XScale0->SetCaptionVisible(1);

	$GraphFrame0->SetViewRange(0, 0, 1, 1);
}

sub AssignGraphData
{
	my ($this, $ResetViewRange) = @_;
	$ResetViewRange = 1 if(!defined $ResetViewRange);

	my $ini = $this->{ini};

	my $GraphFrameArray = $this->GetGraphFrameArray();
	if(!defined $GraphFrameArray) {
		$this->CreateGraphFrame();
		$GraphFrameArray = $this->GetGraphFrameArray();
	}
	my $GraphFrame0  = $GraphFrameArray->GetGraphFrame(0);
	$GraphFrame0->ClearAllData();
	my $GraphFrame1  = $GraphFrameArray->GetGraphFrame(1);
	$GraphFrame1->ClearAllData();

	my $iXLabel  = $this->{"ChooseXListBox"}->GetCurSel();
	$iXLabel = 0 if(!defined $iXLabel);
	my $iY1Label = $this->{"ChooseY1ListBox"}->GetCurSel();
	$iY1Label = 1 if(!defined $iY1Label);
	my $XLabel  = $this->{"ChooseXListBox"}->GetText();
	$XLabel = 'V' if(!defined $XLabel);
	my $Y1Label = $this->{"ChooseY1ListBox"}->GetText();
	$Y1Label = 'I' if(!defined $Y1Label);
	my $pLabelArray = $this->{pLabelArray};
	my $ppDataArray = $this->{ppDataArray};

	my ($xm0, $ym0, $xm1, $ym1) = $GraphFrame0->CalMaxMarginRange();

	my $pX  = $ppDataArray->[$iXLabel];
	my $pY1 = $ppDataArray->[$iY1Label];
	if($pX) {
		my $nData = @$pX;
		my $nDataArray = @$pLabelArray;
		return unless($pX or $pY1);

		$GraphFrame0->SetXCaption($XLabel);
		$GraphFrame0->SetYCaption($Y1Label);

		$GraphFrame1->SetXCaption($XLabel);
		$GraphFrame1->SetYCaption("P(W/cm2)");

		my (@VP, @P);
		my $c = 0;
		for(my $i = 0 ; $i < @$pX ; $i++) {
			my $P  = -$pX->[$i] * $pY1->[$i];
			if($P >= 0.0) {
				$VP[$c]  = $pX->[$i];
				$P[$c++] = $P;
			}
		}

		my $nData0 = $GraphFrame0->AddGraphData($pX, $pY1,
					1, "black", "", 6, "red", 0, "red", "XAutoSkip", $XLabel, $Y1Label);

		my $s = sprintf("%6.4f", $ini->{Voc});
		my $nData2 = $GraphFrame0->AddGraphData([$ini->{Voc}], [0.0],
				0, "", "circle", 6, "red", 3, "red", "XAutoSkip", "Voc", "Voc=$s V");

		$s = sprintf("%6.4f", $ini->{Jsc}*1.0e-3);
		my $nData3 = $GraphFrame0->AddGraphData([0.0], [-$ini->{Jsc}*1.0e-3],
				0, "", "circle", 6, "red", 1, "red", "XAutoSkip", "Jsc", "Jsc=$s A/cm2");

		$s = sprintf("%6.4f, %6.4f", $ini->{Vop}, $ini->{Jop}*1.0e-3);
		my $nData4 = $GraphFrame0->AddGraphData([$ini->{Vop}], [-$ini->{Jop}*1.0e-3],
				0, "", "circle", 6, "red", 1, "red", "XAutoSkip", "Wmax", "Wmax ($s)");

		if(defined $ini->{VforR}) {
			my $V0 = $ini->{VforR};
			my $J0 = $this->{CSV}->YVal($pX, $pY1, $V0);
			my $R  = $ini->{RsOn};
			my $JRsOn = $this->{CSV}->YVal($pX, $pY1, $V0);
			$s = sprintf("Rs(on)=%6.4f ohm cm2 @ %6.4f V", $R, $JRsOn);
			my $nData5 = $GraphFrame0->AddGraphData([$V0], [$JRsOn],
				0, "", "cross", 6, "blue", 1, "blue", "XAutoSkip", "Rs(on)", "$s");

			my (@VRsOn, @JRsOn);
			$c = 0;
			for(my $i = 0 ; $i < @$pX ; $i++) {
				my $J  = ($pX->[$i] - $V0) / $R + $J0;
				if($J >= $ym0 and $J <= $ym1) {
					$VRsOn[$c]   = $pX->[$i];
					$JRsOn[$c++] = $J;
				}
			}
			my $nData5b = $GraphFrame0->AddGraphData(\@VRsOn, \@JRsOn,
				1, "blue", "", 6, "blue", 0, "blue", "XAutoSkip", "Rs(on)", "$s");

			$V0 = $ini->{Voc};
			$J0 = 0.0;
			$R  = $ini->{RsVoc};
			$s = sprintf("Rs(Voc)=%6.4f ohm cm2", $R);
			my (@VRsVoc, @JRsVoc);
			$c = 0;
			for(my $i = 0 ; $i < @$pX ; $i++) {
				my $J  = ($pX->[$i] - $V0) / $R + $J0;
				if($J >= $ym0 and $J <= $ym1) {
					$VRsVoc[$c]   = $pX->[$i];
					$JRsVoc[$c++] = $J;
				}
			}
			my $nData5c = $GraphFrame0->AddGraphData(\@VRsVoc, \@JRsVoc,
				1, "blue", "", 6, "blue", 0, "blue", "XAutoSkip", "Rs(Voc)", "$s");
		}

		if(defined $ini->{VrevR}) {
			my $V0 = $ini->{VrevR};
			my $J0 = $this->{CSV}->YVal($pX, $pY1, $V0);
			my $R  = $ini->{RsRev};
			my $JRsrev = $this->{CSV}->YVal($pX, $pY1, $V0);
			$s = sprintf("Rs(rev)=%6.4f ohm cm2 @ %6.4f V", $R, $V0);
			my $nData6 = $GraphFrame0->AddGraphData([$ini->{VrevR}], [$JRsrev],
				0, "", "cross", 6, "blue", 1, "blue", "XAutoSkip", "Rs(rev)", "$s");

			my (@VRrev, @JRrev);
			$c = 0;
			for(my $i = 0 ; $i < @$pX ; $i++) {
				my $J  = ($pX->[$i] - $V0) / $R + $J0;
				if($J >= $ym0 and $J <= $ym1) {
					$VRrev[$c]   = $pX->[$i];
					$JRrev[$c++] = $J;
				}
			}
			my $nData6b = $GraphFrame0->AddGraphData(\@VRrev, \@JRrev,
				1, "blue", "", 6, "blue", 0, "blue", "XAutoSkip", "Rs(rev)", "$s");

			$V0 = 0.0;
			$J0 = -$ini->{Jsc} * 1.0e-3;
			$R  = $ini->{Rsh0};
			$s = sprintf("Rsh(0)=%6.4f ohm cm2", $R);
			my (@VRsh0, @JRsh0);
			$c = 0;
			for(my $i = 0 ; $i < @$pX ; $i++) {
				my $J  = ($pX->[$i] - $V0) / $R + $J0;
				if($J >= $ym0 and $J <= $ym1) {
					$VRsh0[$c]   = $pX->[$i];
					$JRsh0[$c++] = $J;
				}
			}
			my $nData6c = $GraphFrame0->AddGraphData(\@VRsh0, \@JRsh0,
				1, "blue", "", 6, "blue", 0, "blue", "XAutoSkip", "Rsh(0)", "$s");
		}

		my $nData1 = $GraphFrame1->AddGraphData(\@VP, \@P,
					1, "black", "", 6, "red", 0, "red", "XAutoSkip", $XLabel, "P");

		$s = sprintf("%6.4f V, %6.4f W/cm2", $ini->{Vop}, $ini->{Wmax}*1e-3);
		my $nData5 = $GraphFrame1->AddGraphData([$ini->{Vop}], [$ini->{Wmax}*1e-3],
				0, "", "circle", 6, "red", 1, "red", "XAutoSkip", "Wmax", "Wmax=$s");
	}

	if($this->{ini}->{ShowCalcPlot} and $this->{pIcal}) {
		my $nData1 = $GraphFrame0->AddGraphData($this->{pVcal}, $this->{pIcal},
				1, "red", "", 6, "red", 0, "red", "XAutoSkip", $XLabel, "${Y1Label}(cal)");
	}

	$GraphFrame0->SetYScalePlotType('x');
	$GraphFrame0->CalMinMax();
	$GraphFrame0->AdjustViewRange(0.05, 0.05, 0.05, 0.05);
	$GraphFrame1->SetYScalePlotType('x');
	$GraphFrame1->CalMinMax();
	$GraphFrame1->AdjustViewRange(0.05, 0.05, 0.05, 0.05);
	if($ResetViewRange) {
		$this->{ini}->{FitXMin} = $GraphFrame0->XMin();
		$this->{ini}->{FitXMax} = $GraphFrame0->XMax();
	}
}

sub AdjustViewRange
{
	my ($this) = @_;
	my $GraphFrameArray = $this->GetGraphFrameArray();
	my $pGraphFrame = $GraphFrameArray->GetpGraphFrameArray();
	my $FileType   = $this->FileType();

	if($FileType =~ /(Ids)/i) {
	}
}

sub Draw
{
	my ($this, $canvas, $TargetData) = @_;
	my $this = $this->mw();
	$canvas = $this->Canvas() if(!$canvas);
#	my $FileType   = $this->FileType();
	my $App = $this->App();
	my $font = $App->{'GraphFrameFont'};
	my @font = split(/,/, $font) if($font);
	my $GraphFrameArray = $this->GetGraphFrameArray();
	return unless($GraphFrameArray);

	$canvas->ClearAll();
	$this->RefleshCanvas();

	if($font) {
		$canvas->SetFont(\@font);
		$GraphFrameArray->SetFont(\@font);
	}
	my $w = $canvas->width();
	my $h = $canvas->height();
	$GraphFrameArray->SetCanvasSize($w, $h);

	$this->mw()->WriteStatusBar("Drawing IV...");
#	$this->ReadFiles($this->FileName(), $TargetData);
#	$this->CreateGraphFrame($canvas, $TargetData);
#	$this->AssignGraphData();

	$this->Balloon()->detach($canvas);
	$GraphFrameArray->Draw($canvas);
	$this->mw()->WriteStatusBar("Finish IV.");
}

sub SelChangeListBox
{
	my ($this, $type, $lb) = @_;
#print "this:$this  lb=$lb\n";
	my $s = $lb->GetText();
#	if($s =~ /Solar/i) {
#	}
#	$this->CreateGraphFrame();
	$this->AssignGraphData();
	$this->Draw();
}

sub ChooseFile
{
	my ($this, $type, $widget, $Option) = @_;
	my $App = $this->App();
	my $canvas = $this->Canvas();
	my $this = $this->mw();

	my $mode = 'open';
	my $fmask = '*.csv';
	if($Option eq 'ChooseParameterFile') {
#		$mode = 'save';
		$fmask = '*.prm';
	}

	my $dir = '';
	unless($widget) {
		$dir = $this->App()->{WorkDir};
		chdir($dir);
	}
	my $defstr = '';
	my $message = 'Choose file';
	my $filepath = Dialog::OpenFileDialog($this, 
				$mode, $fmask, $defstr, $message, $dir);
	if($filepath) {
		my $DirPath = $this->SetWorkDir($filepath);
		if($widget) {
			$widget->SetText($filepath);
		}
	}
	else {
		return undef;
	}

	my $ret;
	if($Option eq 'ChooseParameterFile') {
		$this->{ini}->{DataFile} = '';
		$this->{ini}->ReadAll($filepath);
		$this->{ini}->{ParameterFile} = $filepath;
		$this->{ChooseModelListBox}->SetText($this->{ini}->{Model}) 
				if($this->{ChooseModelListBox} and $this->{ini}->{Model});
		$this->{ChooseXListBox}->SetText($this->{ini}->{X}) 
				if($this->{ChooseXListBox} and $this->{ini}->{X});
		$this->{ChooseY1ListBox}->SetText($this->{ini}->{Y1}) 
				if($this->{ChooseY1ListBox} and $this->{ini}->{Y1});

		if($this->{ini}->{DataFile}) {
			$ret = $this->ReadFile($this->{ini}->{DataFile}, 1, 0);
			return undef unless($ret);
			$this->Draw();
		}
	}
	elsif($Option eq 'ChooseCSVFile') {
		$ret = $this->ReadFile($filepath);
		return undef unless($ret);
		$this->Draw();
	}

	return $ret;
}

sub ChooseSaveFile
{
	my ($this, $path, $fmask, $widget) = @_;
	$path = 'save.csv' if(!defined $path);
	$fmask = '*.csv' if(!defined $fmask);

	$path = $widget->GetText() if($widget);
	$path = 'out.csv' if(!$path);
	my $App = $this->App();
	my $this = $this->mw();

	my $dir = $this->App()->{WorkDir};
	chdir($dir) if($dir);

	my $defstr = $path;
	my $message = 'Choose file';
	my $filepath = Dialog::OpenFileDialog($this, 
				'save', $fmask, $defstr, $message, $dir);
	if($filepath) {
		my $DirPath = $this->SetWorkDir($filepath);
		if($widget) {
			$widget->SetText($filepath);
		}
	}
	return $filepath;
}

sub InitializeParameters
{
	my ($this, $ini, $dini) = @_;

#	$this->AddParameters($ini, $dini, "EF",       0.0, 0,    1.0,   '', '');
}

1;