#!/bin/bash

SCRIPTDir=`pwd`/script
VASPDir=$TkPerlDir/VASP
CMD="/usr/bin/perl $VASPDir/VASP.pl"
AddMDHistoryCMD="/usr/bin/perl $SCRIPTDir/AddMDHistory.pl"
VASPCommand=./DoVASP.sh

TemplateDir=`pwd`/Template
HistoryFile=History.csv

post=

TemplateList="6000 5900 5800 5500 5300 5000 4500 4000 3500 3000 2500 2000 1500 1000 900 800 700 500 300 10 last"
NSW=1000
POTIM=2.0

#===========================
# Calculate all
#===========================

OriginalDir=`pwd`

PrevDir=
for Template in $TemplateList
do
	WorkDir=$Template
	echo Working in $WorkDir...

	if [ -f STOP.`hostname` ]; then
		exit -1
	fi

	if [ -d $WorkDir ]; then
		echo [$WorkDir] exists.
		echo Skip.
		continue
	fi

	mkdir $WorkDir
	cp $TemplateDir/{hosts,INCAR,KPOINTS,POSCAR,POTCAR} $WorkDir
	if [ -f $TemplateDir/DoVASP.sh.$Template ]; then
		cp $TemplateDir/DoVASP.sh.$Template $WorkDir/DoVASP.sh
	else
		echo "sed -e "s/NSWMD=/NSW=$NSW/g" $TemplateDir/DoVASP.sh > $WorkDir/DoVASP.sh"
		sed -e "s/^\s*NSWMD\s*=.*$/NSWMD=$NSW/g" \
		    -e "s/^\s*POTIM\s*=.*$/POTIM=$POTIM/g" \
		    -e "s/^\s*KeepSymmetryMD\s*=.*$/KeepSymmetryMD=0/g" $TemplateDir/DoVASP.sh \
		    -e "s/^\s*TEBEG\s*=.*$/TEBEG=$Template/g" \
		    -e "s/^\s*TEEND\s*=.*$/TEEND=$Template/g" $TemplateDir/DoVASP.sh > $WorkDir/DoVASP.sh
	fi
	chmod +x $WorkDir/DoVASP.sh

	if [ "$PrevDir" = "" ]; then
		echo ''
	else
		cp $PrevDir/POSCAR $WorkDir/POSCAR
	fi

	cd $WorkDir
	echo === Working in \"`pwd`\"
	echo === Files in \"$wd\"
	ls

	echo ''
	echo === Execute [$VASPCommand]
	$VASPCommand

	echo ''
	echo === Exit to \"$OriginalDir\"
	cd $OriginalDir

	echo ''
	echo ''
	
	PrevDir=$WorkDir
done

#===========================
# Post processors
#===========================
#$SCRIPTDir/MergePositionsInCIFs.pl --Positions=$iSite 0000.last/SCF MergedCIF.cif
#$SCRIPTDir/MakeSummaryCSV2.pl --Positions=$iSite --AtomType=$AtomType

rm $HistoryFile
for Template in $TemplateList
do
	$SCRIPTDir/AddMDHistory.pl $HistoryFile $Template
done
