#!/bin/bash

SCRIPTDir=`pwd`/script
VASPDir=$TkPerlDir/VASP
CMD="/usr/bin/perl $VASPDir/VASP.pl"

Header=BaSiO3-P63mmc
BaseDir=$Header
TemplateDir=`pwd`/Template-${Header}-Volume
CIFFileName=${Header}.cif

PBE96DoVASP=DoVASP.sh
PBE0DoVASP=DoVASP-HF-mpi.sh
PBE96VASPCommand=./$PBE96DoVASP
PBE0VASPCommand=./$PBE0DoVASP

PBE96DoVASPTemplate=$TemplateDir/DoVASP-Template.sh
PBE0DoVASP=$TemplateDir/DoVASP-HF-mpi.sh

ModifyINCARCommand="/usr/bin/perl $SCRIPTDir/ModifyINCARforPBE0.pl"

#AddMDHistoryCMD="/usr/bin/perl $SCRIPTDir/AddMDHistory.pl"
#HistoryFile=History.csv

post=

#ASiteList="Mg Ca Sr Ba Cd"
#BSiteList="C Si Ge Sn Ti Zr Hf Ce Pt"
PList="100 200 400 500 600 700 800 900 1000 1500 2000 3000 4000 5000 6000 7000 8000 9000 10000 11000 12000 13000 14000 15000 16000 17000 18000 19000 21000 0"

#===========================
# Calculate all
#===========================

OriginalDir=`pwd`

mkdir $BaseDir

PrevDir=
for P in $PList
do
#===============================
# PBE96 calculation
#===============================
	PBE96WorkDir=$BaseDir/${Header}-PBE96-P${P}
	echo Working in $PBE96WorkDir...

	if [ -f STOP.`hostname` ]; then
		exit -1
	fi

	if [ -d $PBE96WorkDir ]; then
		echo [$PBE96WorkDir] exists.
		echo Skip.
		continue
	fi

	mkdir $PBE96WorkDir
	cp $TemplateDir/{${CIFFileName},hosts,*.klist} $PBE96WorkDir

	echo "sed -e \"s/{PStress}/$P/g\" $PBE96DoVASPTemplate \> $PBE96WorkDir/${PBE96DoVASP}"
	sed -e "s/{PStress}/$P/g" $PBE96DoVASPTemplate > $PBE96WorkDir/${PBE96DoVASP}

	chmod +x $PBE96WorkDir/*.sh

	cd $PBE96WorkDir
	echo === Working in \"`pwd`\"
	echo === Files in \"`pwd`\"
	ls

	echo ''
	echo === Execute [$PBE96VASPCommand]
	$PBE96VASPCommand

	echo ''
	echo === Exit to \"$OriginalDir\"
	cd $OriginalDir

continue

#===============================
# PBE0 calculation
#===============================
	PBE0WorkDir=$BaseDir/${Header}-PBE0-P${P}
	echo Working in $PBE0WorkDir...

	if [ -f STOP.`hostname` ]; then
		exit -1
	fi

	if [ -d $PBE0WorkDir ]; then
		echo [$PBE0WorkDir] exists.
		echo Skip.
		continue
	fi

	mkdir $PBE0WorkDir
	cp $PBE96WorkDir/{INCAR,POSCAR,POTCAR,KPOINTS} $PBE0WorkDir
	cp $PBE0TemplateDir/{*.sh,hosts,*.klist} $PBE0WorkDir
	$ModifyINCARCommand $PBE0WorkDir/INCAR

	chmod +x $PBE0WorkDir/*.sh

	cd $PBE0WorkDir
	echo === Working in \"`pwd`\"
	echo === Files in \"$wd\"
	ls

	echo ''
	echo === Execute [$PBE0VASPCommand]
	$PBE0VASPCommand

	echo ''
	echo === Exit to \"$OriginalDir\"
	cd $OriginalDir

	echo ''
	echo ''

	PrevDir=$WorkDir
done

#===========================
# Post processors
#===========================
#$SCRIPTDir/MergePositionsInCIFs.pl --Positions=$iSite 0000.last/SCF MergedCIF.cif
#$SCRIPTDir/MakeSummaryCSV2.pl --Positions=$iSite --AtomType=$AtomType

cd $BaseDir
perl $SCRIPTDir/MakeSummaryCSV.pl TotalEnergy.csv .
cd $OriginalDir

#rm $HistoryFile
#for Template in $TemplateList
#do
#	$AddMDHistoryCMD $HistoryFile $Template
#done
