#!/bin/bash

SCRIPTDir=`pwd`/script
VASPDir=$TkPerlDir/VASP
CMD="/usr/bin/perl $VASPDir/VASP.pl"
AddMDHistoryCMD="/usr/bin/perl $SCRIPTDir/AddMDHistory.pl"
VASPCommand=./DoVASP.sh

TemplateDir=`pwd`/Template-PBE96
TemplatePath=$TemplateDir/Template-cubic-Perovskite.cif
HistoryFile=History.csv

# Set SkipIfDirectoryExists=1 if you do not want to repeat finished tasks
SkipIfDirectoryExists=0

post=

ASiteList="Mg Ca Sr Ba Cd"
BSiteList="C Si Ge Sn Ti Zr Hf"

#===========================
# Calculate all
#===========================

OriginalDir=`pwd`

PrevDir=
for A in $ASiteList
do
for B in $BSiteList
do
	WorkDir=${A}${B}O3-cubic-PBE96
	echo Working in $WorkDir...

	if [ -f STOP.`hostname` ]; then
		exit -1
	fi

	if [ "$SkipIfDirectoryExists" = "1" -a -d $WorkDir ]; then
		echo [$WorkDir] exists.
		echo Skip.
		continue
	fi

	mkdir $WorkDir
	cp $TemplateDir/{*.sh,hosts,*.klist} $WorkDir

	echo "sed -e \"s/{A}/$A/g\"-e \"s/{B}/$B/g\" $TemplatePath \> cif.temp"
	sed -e "s/{A}/$A/g" -e "s/{B}/$B/g" $TemplatePath > $WorkDir/${A}${B}O3-cubic.cif

	chmod +x $WorkDir/DoVASP.sh

	cd $WorkDir
	echo === Working in \"`pwd`\"
	echo === Files in \"$wd\"
	ls

	echo ''
	echo === Execute [$VASPCommand]
	$VASPCommand

	echo ''
	echo === Exit to \"$OriginalDir\"
	cd $OriginalDir

	echo ''
	echo ''

	PrevDir=$WorkDir
done
done

#===========================
# Post processors
#===========================
#$SCRIPTDir/MergePositionsInCIFs.pl --Positions=$iSite 0000.last/SCF MergedCIF.cif
#$SCRIPTDir/MakeSummaryCSV2.pl --Positions=$iSite --AtomType=$AtomType

#rm $HistoryFile
#for Template in $TemplateList
#do
#	$AddMDHistoryCMD $HistoryFile $Template
#done
